@@ -28,6 +28,7 @@ def _gtk_init(icon=None):
     import pygtk
     pygtk.require('2.0')
     import gtk
+    import gobject
 
     # http://bugzilla.gnome.org/show_bug.cgi?id=318953
     if gtk.gtk_version < (2, 8, 8):
@@ -38,6 +39,10 @@ def _gtk_init(icon=None):
     import quodlibet.stock
     quodlibet.stock.init()
 
+    gobject.set_application_name(_("Quod Libet").encode('utf-8'))
+    os.environ["PULSE_PROP_media.role"] = "music"
+    os.environ["PULSE_PROP_application.icon_name"] = quodlibet.stock.QL_ICON
+
     if icon:
         icon = os.path.join(quodlibet.const.IMAGEDIR, icon)
         try: gtk.window_set_default_icon_from_file(icon + ".svg")
@@ -58,10 +63,8 @@ def print_(string, frm="utf-8", prefix="", output=sys.stdout, log=None):
 
     quodlibet.util.logging.log(string, log)
 
-    if isinstance(string, unicode):
-        string = string.encode(ENCODING, "replace")
-    else:
-        string = string.decode(frm).encode(ENCODING, "replace")
+    if os.name == 'nt':
+        return
 
     if output:
         if isinstance(string, unicode):
@@ -95,13 +98,13 @@ def print_w(string):
     """Print warnings."""
     # Translators: "W" as in "Warning". It is prepended to
     # terminal output. APT uses a similar output format.
-    print_(string, prefix=_("W: "), log="Warnings", output=sys.stderr)
+    print_(string, prefix=_("W: "), log=_("Warnings"), output=sys.stderr)
 
 def print_e(string):
     """Print errors."""
     # Translators: "E" as in "Error". It is prepended to
     # terminal output. APT uses a similar output format.
-    print_(string, prefix=_("E: "), log="Errors", output=sys.stderr)
+    print_(string, prefix=_("E: "), log=_("Errors"), output=sys.stderr)
 
 def _python_init():
     # The default regex escaping function doesn't work for non-ASCII.
@@ -121,8 +124,10 @@ def _python_init():
             tb = traceback.extract_stack()
             for (filename, linenum, func, text) in tb:
                 if "plugins" in filename:
-                    warnings.warn(
-                        "enabling legacy plugin API", DeprecationWarning)
+                    # warn() causes errors when running from py2exe
+                    if not (os.name == 'nt' and 'library.zip' in __file__):
+                        warnings.warn(
+                            "enabling legacy plugin API", DeprecationWarning)
                     old_import("quodlibet." + module, *args, **kwargs)
                     return sys.modules["quodlibet." + module]
             else:
@@ -193,8 +198,8 @@ def main(window):
     print_d("Entering quodlibet.main")
     import gtk
 
-    SIGNALS = [signal.SIGINT, signal.SIGTERM, signal.SIGHUP]
-    for sig in SIGNALS:
+    SIGS = [getattr(signal, s, None) for s in "SIGINT SIGTERM SIGHUP".split()]
+    for sig in filter(None, SIGS):
         signal.signal(sig, window.destroy)
 
     window.connect('destroy', gtk.main_quit)
@@ -203,7 +208,9 @@ def main(window):
     # This has been known to cause segmentation faults in some Python,
     # GTK+, and GStreamer versions.
     gtk.gdk.threads_init()
+    gtk.gdk.threads_enter()
     gtk.main()
+    gtk.gdk.threads_leave()
 
 def error_and_quit(error):
     from quodlibet.qltk.msg import ErrorMessage
