@@ -4,14 +4,12 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: __init__.py 1c3d0b94507e 2007/05/29 01:00:53 $
 
 import os
 import sys
-import traceback
 
 from quodlibet import const
+from quodlibet import util
 
 from os.path import dirname, basename, isdir, join
 from glob import glob
@@ -21,6 +19,9 @@ from quodlibet.browsers._base import Browser
 BROWSERS = os.path.join(const.USERDIR, "browsers")
 
 base = dirname(__file__)
+if os.name == 'nt' and 'library.zip' in base:
+    # running a py2exe version, use the alternate browser location
+    base = os.path.join(const.BASEDIR, 'browsers')
 self = basename(base)
 parent = basename(dirname(base))
 modules = [f[:-3] for f in glob(join(base, "[!_]*.py"))]
@@ -46,7 +47,7 @@ browsers = []
 for name in modules:
     try: browser = __import__(name, {}, {}, self)
     except Exception, err:
-        traceback.print_exc()
+        util.print_exc()
         continue
 
     try: browsers.extend(browser.browsers)
