@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: _base.py 862a0356e3ef 2007/08/19 21:00:47 $
 
 import gobject
 import gtk
@@ -87,6 +85,10 @@ class Browser(object):
     def save(self): raise NotImplementedError
     def restore(self): raise NotImplementedError
 
+    # Called after restore/activate or after the browser is loaded.
+    # restored is True if restore was called.
+    def finalize(self, restored): pass
+
     # Decides whether "filter on foo" menu entries are available.
     def can_filter(self, key): return False
 
@@ -119,7 +121,8 @@ class Browser(object):
     # menu is returned the SongList may modify it further.
     def Menu(self, songs, songlist, library):
         menu = SongsMenu(
-            library, songs, delete=True, accels=songlist.accelerators)
+            library, songs, delete=True, accels=songlist.accelerators,
+            parent=self)
         return menu
 
     def statusbar(self, i):
