@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
+# Copyright 2004-2010 Joe Wreschnig, Michael Urman, Iñigo Serna,
+#                     Christoph Reiter, Steven Robertson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: albums.py 7fb5bf908e15 2009/06/22 01:57:25 $
 
 import os
 
@@ -20,22 +19,24 @@ from quodlibet import stock
 from quodlibet import util
 
 from quodlibet.browsers._base import Browser
-from quodlibet.formats._audio import PEOPLE, PEOPLE_SORT
+from quodlibet.browsers.search import BoxSearchBar
+from quodlibet.formats._audio import PEOPLE, TAG_TO_SORT
 from quodlibet.parse import Query, XMLFromPattern
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.completion import EntryWordCompletion
-from quodlibet.qltk.entry import ValidatingEntry
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.textedit import PatternEditBox
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.util import copool
+from quodlibet.util import thumbnails
 
-ELPOEP = list(PEOPLE); ELPOEP.reverse()
-ELPOEPSORT = list(PEOPLE_SORT); ELPOEPSORT.reverse()
+ELPOEP = list(reversed(PEOPLE))
+PEOPLE_SCORE = [100**w for w in xrange(len(PEOPLE))]
 EMPTY = _("Songs not in an album")
 PATTERN = r"""\<b\><title|\<i\><title>\</i\>|%s>\</b\><date| (<date>)>
 \<small\><~discs|<~discs> - ><~tracks> - <~long-length>\</small\>
 <people>""" % EMPTY
+ALBUM_QUERIES = os.path.join(const.USERDIR, "lists", "album_queries")
 
 class AlbumTagCompletion(EntryWordCompletion):
     def __init__(self):
@@ -56,14 +57,15 @@ class AlbumTagCompletion(EntryWordCompletion):
             for suffix in ["avg", "max", "min", "sum"]:
                 self.__model.append(row=["#(%s:%s" % (tag, suffix)])
 
-class Preferences(qltk.Window):
-    def __init__(self):
+class Preferences(qltk.UniqueWindow):
+    def __init__(self, parent):
+        if self.is_not_unique(): return
         super(Preferences, self).__init__()
         self.set_border_width(12)
         self.set_title(_("Album List Preferences") + " - Quod Libet")
         self.add(gtk.VBox(spacing=6))
-        self.set_default_size(300, 200)
-        self.connect_object('delete-event', Preferences.__delete_event, self)
+        self.set_default_size(300, 250)
+        self.set_transient_for(qltk.get_top_parent(parent))
 
         cb = ConfigCheckButton(
             _("Show album _covers"), "browsers", "album_covers")
@@ -71,6 +73,12 @@ class Preferences(qltk.Window):
         cb.connect('toggled', lambda s: AlbumList.toggle_covers())
         self.child.pack_start(cb, expand=False)
 
+        cb = ConfigCheckButton(
+            _("Inline _search includes people"),
+            "browsers", "album_substrings")
+        cb.set_active(config.getboolean("browsers", "album_substrings"))
+        self.child.pack_start(cb, expand=False)
+
         vbox = gtk.VBox(spacing=6)
         label = gtk.Label()
         label.set_alignment(0.0, 0.5)
@@ -85,11 +93,7 @@ class Preferences(qltk.Window):
         f = qltk.Frame(_("Album Display"), child=vbox)
         self.child.pack_start(f)
 
-        self.child.show_all()
-
-    def __delete_event(self, event):
-        self.hide()
-        return True
+        self.show_all()
 
     def __set_pattern(self, apply, edit):
         AlbumList.refresh_pattern(edit.text)
@@ -144,6 +148,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         on = config.getboolean("browsers", "album_covers")
         for albumlist in klass.instances():
             albumlist.__cover_column.set_visible(on)
+            albumlist.__column.queue_resize()
 
     @classmethod
     def refresh_pattern(klass, pattern_text):
@@ -221,8 +226,6 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
 
     # Something like an AudioFile, but for a whole album.
     class _Album(object):
-        __pending_covers = []
-
         _pattern_text = PATTERN
         _pattern = XMLFromPattern(PATTERN)
 
@@ -231,6 +234,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         tracks = 0
         date = ""
         markup = ""
+        scanned = False
 
         def __init__(self, title, sort, labelid, mbid):
             self.people = []
@@ -244,6 +248,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             # but the same label ID are the same (since MB uses separate
             # MBIDs for each disc).
             self.key = (sort, labelid or mbid)
+            self.human = util.human_sort_key(sort)
 
         def get(self, key, default="", connector=u" - "):
             if "~" in key[1:]:
@@ -294,9 +299,26 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             return self.get(*args).replace("\n", ", ")
 
         def refresh(self):
-            if self._model:
+            if self._model and self._iter:
                 self._model[self._iter][0] = self
 
+        def scan_cover(self):
+            if self.scanned or not self.songs: return
+            self.scanned = True
+
+            song = list(self.songs)[0]
+            cover = song.find_cover()
+
+            if cover is not None:
+                try:
+                    round = config.getboolean("settings", "round")
+                    self.cover = thumbnails.get_thumbnail(cover.name, (48, 48))
+                    self.cover = thumbnails.add_border(self.cover, 30, round)
+                except gobject.GError:
+                    return
+                else:
+                    self.refresh()
+
         # All songs added, cache info.
         def finalize(self, cover=True):
             self.tracks = len(self.songs)
@@ -307,11 +329,16 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                 # Rank people by "relevance" -- artists before composers
                 # before performers, then by number of appearances.
                 for w, key in enumerate(ELPOEP):
-                    for person in song.list(key):
-                        people[person] = people.get(person, 0) - 100 ** w
-                for w, key in enumerate(ELPOEPSORT):
-                    for person in (song.list(key) or song.list(ELPOEP[w])):
-                        peoplesort[person] = peoplesort.get(person, 0) - 100**w
+                    persons = song.list(key)
+                    for person in persons:
+                        people[person] = (people.get(person, 0) -
+                                          PEOPLE_SCORE[w])
+                    if key in TAG_TO_SORT:
+                        persons = song.list(TAG_TO_SORT[key]) or persons
+                    for person in persons:
+                        person = util.human_sort_key(person)
+                        peoplesort[person] = (peoplesort.get(person, 0) -
+                                              PEOPLE_SCORE[w])
 
                 self.discs = max(self.discs, song("~#disc", 0))
                 self.length += song.get("~#length", 0)
@@ -329,48 +356,26 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             self.refresh()
 
         def remove(self, song):
-            try: self.songs.remove(song)
-            except KeyError: return False
-            else: return True
+            if song in self.songs:
+                self.songs.remove(song)
+                return True
+            else: return False
 
-    # An auto-searching entry; it wraps is a TreeModelFilter whose parent
-    # is the album list.
-    class FilterEntry(ValidatingEntry):
-        def __init__(self, model):
-            ValidatingEntry.__init__(self, Query.is_valid_color)
-            self.connect_object('changed', self.__filter_changed, model)
-            self.set_completion(AlbumTagCompletion())
-            self.__refill_id = None
-            self.__filter = None
-            self.inhibit = False
-            model.set_visible_func(self.__parse)
+    class FilterBar(BoxSearchBar):
+        """The search filter entry HBox, modifiedto toggle between the search
+        bar and album list on mnemonic activation."""
+        def __init__(self, albumlist, *args, **kwargs):
+            super(AlbumList.FilterBar, self).__init__(*args, **kwargs)
+            self.albumlist = albumlist
 
-        def __parse(self, model, iter):
-            if self.__filter is None: return True
-            elif model[iter][0] is None: return True
-            else: return self.__filter(model[iter][0])
+        def _mnemonic_activate(self, label, group_cycling):
+            widget = label.get_mnemonic_widget()
+            if widget.is_focus():
+                self.albumlist._get_view().grab_focus()
+                return True
 
-        def __filter_changed(self, model):
-            if self.__refill_id is not None:
-                gobject.source_remove(self.__refill_id)
-                self.__refill_id = None
-            text = self.get_text().decode('utf-8')
-            if Query.is_parsable(text):
-                if not text: self.__filter = None
-                else:
-                    self.__filter = Query(
-                        text, star=["people", "album"]).search
-                self.__refill_id = gobject.timeout_add(
-                    500, self.__refilter, model)
-
-        def __refilter(self, model):
-            self.inhibit = True
-            model.refilter()
-            self.inhibit = False
-
-    # Sorting, either by people or album title. It wraps a TreeModelSort
-    # whose parent is the album list.
     class SortCombo(gtk.ComboBox):
+        """ComboBox which sets the sort function on a TreeModelSort."""
         def __init__(self, model):
             # Contains string to display, function to do sorting
             cbmodel = gtk.ListStore(str)
@@ -382,9 +387,8 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             model.set_sort_func(101, self.__compare_artist)
             model.set_sort_func(102, self.__compare_date)
 
-            for text in [
-                _("Sort by title"), _("Sort by artist"), _("Sort by date")
-                ]: cbmodel.append(row=[text])
+            for text in [_("Title"), _("Artist"), _("Date")]:
+                cbmodel.append(row=[text])
 
             self.connect_object('changed', self.__set_cmp_func, model)
             try: active = config.getint('browsers', 'album_sort')
@@ -401,7 +405,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             if (a1 and a2) is None: return cmp(a1, a2)
             elif not a1.sort: return 1
             elif not a2.sort: return -1
-            else: return cmp(a1.key, a2.key)
+            else: return cmp((a1.human, a1.key), (a2.human, a2.key))
 
         def __compare_artist(self, model, i1, i2):
             a1, a2 = model[i1][0], model[i2][0]
@@ -413,7 +417,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             else: return (cmp(a1.peoplesort and a1.peoplesort[0],
                               a2.peoplesort and a2.peoplesort[0]) or
                           cmp(a1.date or "ZZZZ", a2.date or "ZZZZ") or
-                          cmp(a1.key, a2.key))
+                          cmp((a1.human, a1.key), (a2.human, a2.key)))
 
         def __compare_date(self, model, i1, i2):
             a1, a2 = model[i1][0], model[i2][0]
@@ -422,14 +426,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             elif not a2.sort: return -1
             elif not a1.date and a2.date: return 1
             elif not a2.date and a1.date: return -1
-            else: return (cmp(a1.date, a2.date) or cmp(a1.key, a2.key))
+            else: return (cmp(a1.date, a2.date) or
+                cmp((a1.human, a1.key), (a2.human, a2.key)))
 
     class _AlbumStore(gtk.ListStore):
         __gsignals__ = { "row-changed": "override" }
 
         def __init__(self, *args, **kwargs):
             super(AlbumList._AlbumStore, self).__init__(*args, **kwargs)
-            self.__pending_covers = []
 
         def do_row_changed(self, path, iter):
             album = self[iter][0]
@@ -437,34 +441,6 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                 return
             album._model = self
             album._iter = iter
-            if album.title and album.cover is type(album).cover:
-                if not self.__pending_covers:
-                    copool.add(self.__scan_covers)
-                self.__pending_covers.append(album)
-
-        def __scan_covers(self):
-            while self.__pending_covers:
-                album = self.__pending_covers.pop()
-                if album._iter is None or album.cover is not type(album).cover:
-                    continue
-                song = list(album.songs)[0]
-                cover = song.find_cover()
-                if cover is not None:
-                    try:
-                        cover = gtk.gdk.pixbuf_new_from_file_at_size(
-                            cover.name, 48, 48)
-                    except StandardError:
-                        continue
-                    else:
-                        # add a black outline
-                        w, h = cover.get_width(), cover.get_height()
-                        newcover = gtk.gdk.Pixbuf(
-                            gtk.gdk.COLORSPACE_RGB, True, 8, w + 2, h + 2)
-                        newcover.fill(0x000000ff)
-                        cover.copy_area(0, 0, w, h, newcover, 1, 1)
-                        album.cover = newcover
-                        album.refresh()
-                yield True
 
         def get_albums(self):
             albums = [row[0] for row in self]
@@ -486,6 +462,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         model_sort = gtk.TreeModelSort(self.__model)
         model_filter = model_sort.filter_new()
 
+        self.__filter = None
+        model_filter.set_visible_func(self.__parse_query)
+
+        self.__pending_covers = []
+        self.__scan_timeout = None
+        view.connect_object('expose-event', self.__update_visibility, view)
+        sw.get_vadjustment().connect("value-changed", self.__stop_cover_update)
+
         render = gtk.CellRendererPixbuf()
         self.__cover_column = column = gtk.TreeViewColumn("covers", render)
         column.set_visible(config.getboolean("browsers", "album_covers"))
@@ -504,7 +488,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         view.append_column(column)
 
         render = gtk.CellRendererText()
-        column = gtk.TreeViewColumn("albums", render)
+        self.__column = column = gtk.TreeViewColumn("albums", render)
         render.set_property('ellipsize', pango.ELLIPSIZE_END)
         def cell_data(column, cell, model, iter):
             album = model[iter][0]
@@ -522,15 +506,13 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         view.set_rules_hint(True)
         view.set_search_equal_func(self.__search_func)
         view.set_search_column(0)
+        view.set_model(model_filter)
         sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
         sw.add(view)
-        e = self.FilterEntry(model_filter)
-        hb2 = gtk.HBox()
-        hb2.pack_start(e)
-        hb2.pack_start(qltk.ClearButton(e), expand=False)
 
         if player: view.connect('row-activated', self.__play_selection, player)
-        view.get_selection().connect('changed', self.__selection_changed, e)
+        self.__sig = view.get_selection().connect('changed',
+            self.__selection_changed)
 
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
                    ("text/uri-list", 0, 2)]
@@ -539,29 +521,133 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         view.connect("drag-data-get", self.__drag_data_get)
         view.connect_object('popup-menu', self.__popup, view, library)
 
-        hb = gtk.HBox(spacing=6)
-        hb.pack_start(self.SortCombo(model_sort), expand=False)
-        hb.pack_start(hb2)
+        search = AlbumList.FilterBar(
+                self, library, button=False, completion=AlbumTagCompletion())
+        search.callback = self.__update_filter
         prefs = gtk.Button()
-        prefs.add(
-            gtk.image_new_from_stock(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
-        prefs.connect('clicked', self.__preferences)
-        hb.pack_start(prefs, expand=False)
-        self.pack_start(hb, expand=False)
+        prefs.add(gtk.image_new_from_stock(
+            gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
+        prefs.connect('clicked', Preferences)
+        search.pack_start(prefs, expand=False)
+        self.pack_start(search, expand=False)
         self.pack_start(sw, expand=True)
-        view.set_model(model_filter)
+
+        hb = gtk.HBox(spacing=6)
+        l = gtk.Label(_("Sort _by:"))
+        l.set_use_underline(True)
+        sc = self.SortCombo(model_sort)
+        l.set_mnemonic_widget(sc)
+        hb.pack_start(l, expand=False)
+        hb.pack_start(sc)
+        self.pack_start(hb, expand=False)
+
         self.show_all()
 
+    def _get_view(self):
+        return self.get_children()[1].child
+
+    def __update_visible_covers(self, view):
+        vrange = view.get_visible_range()
+        if vrange is None: return
+
+        model_filter = view.get_model()
+        model = model_filter.get_model()
+
+        #generate a path list so that cover scanning starts in the middle
+        #of the visible area and alternately moves up and down
+        preload_count = 20
+
+        start, end = vrange
+        start = start[0] - preload_count - 1
+        end = end[0] + preload_count
+
+        vlist = range(end, start, -1)
+        top = vlist[:len(vlist)/2]
+        bottom = vlist[len(vlist)/2:]
+        top.reverse()
+
+        vlist_new = []
+        for i in vlist:
+            if top: vlist_new.append(top.pop())
+            if bottom: vlist_new.append(bottom.pop())
+        vlist_new = filter(lambda s: s >= 0, vlist_new)
+
+        visible_albums = []
+        for path in vlist_new:
+            model_path = model_filter.convert_path_to_child_path(path)
+            try:
+                iter = model.get_iter(model_path)
+            except TypeError:
+                continue
+            album = model.get_value(iter, 0)
+            if album is not None and not album.scanned:
+                visible_albums.append(album)
+
+        if not self.__pending_covers and visible_albums:
+            copool.add(self.__scan_covers)
+        self.__pending_covers = visible_albums
+
+    def __scan_covers(self):
+        while self.__pending_covers:
+            album = self.__pending_covers.pop()
+            album.scan_cover()
+            yield True
+
+    def __stop_cover_update(self, *args):
+        self.__pending_covers = []
+
+    def __update_visibility(self, view, *args):
+        if not self.__cover_column.get_visible():
+            return
+
+        if self.__scan_timeout:
+            gobject.source_remove(self.__scan_timeout)
+            self.__scan_timeout = None
+
+        self.__scan_timeout = gobject.timeout_add(
+            50, self.__update_visible_covers, view)
+
+    def __update_filter(self, text):
+        view = self._get_view()
+        model = view.get_model()
+        if Query.is_parsable(text):
+            if not text:
+                self.__filter = None
+            else:
+                self.__filter = Query(text, star=["people", "album"]).search
+        self.__inhibit()
+        model.refilter()
+        self.__uninhibit()
+
+    def __parse_query(self, model, iter):
+        if self.__filter is None: return True
+        elif model[iter][0] is None: return True
+        else: return self.__filter(model[iter][0])
+
     def __search_func(self, model, column, key, iter):
-        try: value = model[iter][0].title
-        except AttributeError: return True
+        if config.getboolean("browsers", "album_substrings"):
+            key = key.lower()
+            vals = []
+            try: vals.append(model[iter][0].title.lower())
+            except AttributeError: pass
+            try:
+                for person in model[iter][0].people:
+                    vals.append(person.lower())
+            except AttributeError: pass
+            for val in vals:
+                if key in val: return False
+            return True
         else:
-            key = key.decode('utf-8')
-            return not (value.startswith(key) or value.lower().startswith(key))
-        
+            try: value = model[iter][0].title
+            except AttributeError: return True
+            else:
+                key = key.decode('utf-8')
+                return not (value.startswith(key) or
+                            value.lower().startswith(key))
+
     def __popup(self, view, library):
         songs = self.__get_selected_songs(view.get_selection())
-        menu = SongsMenu(library, songs)
+        menu = SongsMenu(library, songs, parent=self)
 
         button = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
         button.connect('activate', self.__refresh_album, view)
@@ -578,7 +664,8 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             albums = [row[0] for row in model]
         for album in albums:
             if album:
-                album.cover = type(album).cover
+                album.scanned = False
+                album.scan_cover()
                 album.finalize()
 
     def __get_selected_albums(self, selection):
@@ -590,7 +677,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         if None in albums: return None
         else: return albums
 
-    def __get_selected_songs(self, selection):
+    def __get_selected_songs(self, selection, sort=True):
         if not selection:
             return []
         model, rows = selection.get_selected_rows()
@@ -601,8 +688,12 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         # Sort first by how the albums appear in the model itself,
         # then within the album using the default order.
         songs = []
-        for album in albums:
-            songs.extend(sorted(album.songs))
+        if sort:
+            for album in albums:
+                songs.extend(sorted(album.songs))
+        else:
+            for album in albums:
+                songs.extend(album.songs)
         return songs
 
     def __drag_data_get(self, view, ctx, sel, tid, etime):
@@ -611,24 +702,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             filenames = [song("~filename") for song in songs]
             sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
         else: sel.set_uris([song("~uri") for song in songs])
-        
+
     def __play_selection(self, view, indices, col, player):
         player.reset()
 
-    def __preferences(self, button):
-        try: prefs = self.__prefs_win
-        except AttributeError: prefs = self.__prefs_win = Preferences()
-        win = qltk.get_top_parent(self)
-        top, left = win.get_position()
-        w, h = win.get_size()
-        dw, dh = prefs.get_size()
-        prefs.move((left + w // 2) - dw // 2, (top + h // 2) - dh // 2)
-        prefs.present()
-
     def filter(self, key, values):
         assert(key == "album")
         if not values: values = [""]
-        view = self.get_children()[1].child
+        view = self._get_view()
         selection = view.get_selection()
         selection.unselect_all()
         model = view.get_model()
@@ -643,30 +724,40 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             view.scroll_to_cell(first, use_align=True, row_align=0.5)
 
     def unfilter(self):
-        view = self.get_children()[1].child
+        view = self._get_view()
         selection = view.get_selection()
         selection.unselect_all()
         selection.select_path((0,))
 
     def activate(self):
-        self.get_children()[1].child.get_selection().emit('changed')
+        view = self._get_view()
+        view.get_selection().emit('changed')
 
     def can_filter(self, key):
         return (key == "album")
 
     def list(self, key):
         assert (key == "album")
-        view = self.get_children()[1].child
+        view = self._get_view()
         model = view.get_model()
         return [row[0].title for row in model if row[0]]
 
+    def __inhibit(self):
+        view = self._get_view()
+        view.get_selection().handler_block(self.__sig)
+
+    def __uninhibit(self):
+        view = self._get_view()
+        view.get_selection().handler_unblock(self.__sig)
+
     def restore(self):
         albums = config.get("browsers", "albums").split("\n")
-        view = self.get_children()[1].child
+        view = self._get_view()
         selection = view.get_selection()
         # FIXME: If albums is "" then it could be either all albums or
         # no albums. If it's "" and some other stuff, assume no albums,
         # otherwise all albums.
+        self.__inhibit()
         selection.unselect_all()
         if albums == [""]:  selection.select_path((0,))
         else:
@@ -679,36 +770,36 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
 
             if first:
                 view.scroll_to_cell(first[0], use_align=True, row_align=0.5)
+        self.__uninhibit()
 
     def scroll(self, song):
-        view = self.get_children()[1].child
+        view = self._get_view()
         model = view.get_model()
-        values = song.list("album")
         album_key = song.album_key
         for row in model:
             if row[0] is not None and row[0].key == album_key:
                 view.scroll_to_cell(row.path[0], use_align=True, row_align=0.5)
                 sel = view.get_selection()
-                if row.path not in sel.get_selected_rows()[1]:
-                    sel.unselect_all()
-                    sel.select_path(row.path[0])
+                sel.unselect_all()
+                sel.select_path(row.path[0])
                 break
 
-    def __selection_changed(self, selection, sort):
-        if sort.inhibit: return
+    def __selection_changed(self, selection):
         # Without this delay, GTK+ seems to sometimes call this function
         # before model elements are totally filled in, leading to errors
         # like "TypeError: unknown type (null)".
-        gobject.idle_add(self.__update_songs, selection.get_tree_view(), sort)
+        gobject.idle_add(self.__update_songs, selection.get_tree_view())
 
-    def __update_songs(self, view, sort):
+    def __update_songs(self, view):
         selection = view.get_selection()
-        songs = self.__get_selected_songs(selection)
+        songs = self.__get_selected_songs(selection, False)
         albums = self.__get_selected_albums(selection)
-        if not songs: return
+        if not songs:
+            return
         self.emit('songs-selected', songs, None)
         if self.__save:
-            if albums is None: config.set("browsers", "albums", "")
+            if albums is None:
+                config.set("browsers", "albums", "")
             else:
                 confval = "\n".join([a.title for a in albums])
                 # Since ConfigParser strips a trailing \n...
