@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: filesystem.py ffb94ed0ec0b 2009/06/22 02:15:48 $
 
 # Some sort of crazy directory-based browser. QL is full of minor hacks
 # to support this by automatically adding songs to the library when it
@@ -20,6 +18,7 @@ from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
 
+from quodlibet import util
 from quodlibet.browsers._base import Browser
 from quodlibet.library import SongFileLibrary
 from quodlibet.qltk.filesel import DirectoryTree
@@ -63,7 +62,7 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         if "/" not in folders:
             folders.append("/")
 
-        dt = DirectoryTree(initial=const.HOME, folders=folders)
+        dt = DirectoryTree(folders=folders)
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
                    ("text/uri-list", 0, 2)]
         dt.drag_source_set(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
@@ -107,23 +106,32 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         self.child.get_selection().unselect_all()
         for v in values: self.child.go_to(v)
 
+    def scroll(self, song):
+        self.__select_paths([song("~dirname")])
+
     def restore(self):
         try:
             paths = config.get("browsers", "filesystem").split("\n")
-        except StandardError:
+        except config.error:
             pass
         else:
-            def select(model, path, iter, paths):
-                if model[iter][0] in paths:
-                    self.child.get_selection().select_path(path)
-                    paths.remove(model[iter][0])
-                else:
-                    for fpath in paths:
-                        if model[path][0] and fpath.startswith(model[path][0]):
-                            self.child.expand_row(path, False)
-                return not bool(paths)
-            self.child.get_selection().unselect_all()
-            self.child.get_model().foreach(select, paths)
+            self.__select_paths(paths)
+
+    def __select_paths(self, paths):
+        def select(model, path, iter, (paths, first)):
+            if model[iter][0] in paths:
+                self.child.get_selection().select_path(path)
+                paths.remove(model[iter][0])
+                if not first: first.append(path)
+            else:
+                for fpath in paths:
+                    if model[path][0] and fpath.startswith(model[path][0]):
+                        self.child.expand_row(path, False)
+            return not bool(paths)
+        first = []
+        self.child.get_selection().unselect_all()
+        self.child.get_model().foreach(select, (paths, first))
+        if first: self.child.scroll_to_cell(first[0], None, True, 0.5)
 
     def save(self):
         model, rows = self.child.get_selection().get_selected_rows()
@@ -135,7 +143,8 @@ class FileSystem(Browser, gtk.ScrolledWindow):
 
     def Menu(self, songs, songlist, library):
         menu = SongsMenu(library, songs, remove=self.__remove_songs,
-                         delete=True, accels=songlist.accelerators)
+                         delete=True, accels=songlist.accelerators,
+                         parent=self)
         i = qltk.MenuItem(_("_Add to Library"), gtk.STOCK_ADD)
         i.set_sensitive(False)
         i.connect('activate', self.__add_songs, songs)
@@ -162,7 +171,8 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         to_add = []
         for dir in dirs:
             try:
-                for file in filter(formats.filter, sorted(os.listdir(dir))):
+                for file in filter(formats.filter,
+                                   sorted(os.listdir(util.fsnative(dir)))):
                     fn = os.path.realpath(os.path.join(dir, file))
                     if fn in self.__glibrary:
                         songs.append(self.__glibrary[fn])
@@ -171,14 +181,13 @@ class FileSystem(Browser, gtk.ScrolledWindow):
                         if song:
                             to_add.append(song)
                             songs.append(song)
+                            yield songs
                     if fn in self.__library:
                         song = self.__library[fn]
                         if not song.valid():
                             self.__library.reload(song)
                         if song in self.__library:
                             songs.append(song)
-                if not len(to_add) & 0x7:
-                    yield songs
             except OSError: pass
         self.__library.add(to_add)
         yield songs
