@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: playlists.py 0cdf8a58f51e 2009/07/02 06:46:07 $
 
 import os
 import urllib
@@ -90,8 +88,8 @@ def __ParsePlaylist(name, plfilename, files, library):
     return playlist
 
 class Playlist(list):
-    quote = staticmethod(lambda text: urllib.quote(text, safe=""))
-    unquote = staticmethod(urllib.unquote)
+    quote = staticmethod(util.escape_filename)
+    unquote = staticmethod(util.unescape_filename)
 
     def new(klass, base=_("New Playlist"), library={}):
         p = Playlist("", library=library)
@@ -121,12 +119,13 @@ class Playlist(list):
 
     def __init__(self, name, library=None):
         super(Playlist, self).__init__()
-        if isinstance(name, unicode): name = name.encode('utf-8')
+        if isinstance(name, unicode) and os.name != "nt":
+            name = name.encode('utf-8')
         self.name = name
         basename = self.quote(name)
         try:
             for line in file(os.path.join(PLAYLISTS, basename), "r"):
-                line = line.rstrip()
+                line = util.fsnative(line.rstrip())
                 if line in library:
                     self.append(library[line])
                 elif library and library.masked(line):
@@ -188,7 +187,7 @@ class Playlist(list):
         basename = self.quote(self.name)
         f = file(os.path.join(PLAYLISTS, basename), "w")
         for song in self:
-            try: f.write(song("~filename") + "\n")
+            try: f.write(util.fsencode(song("~filename")) + "\n")
             except TypeError: f.write(song + "\n")
         f.close()
 
@@ -249,7 +248,7 @@ class Playlists(gtk.VBox, Browser):
 
     def init(klass, library):
         model = klass.__lists.get_model()
-        for playlist in os.listdir(PLAYLISTS):
+        for playlist in os.listdir(util.fsnative(PLAYLISTS)):
             try:
                 playlist = Playlist(Playlist.unquote(playlist), library=library)
                 model.append(row=[playlist])
@@ -358,7 +357,10 @@ class Playlists(gtk.VBox, Browser):
                    ("text/x-moz-url", 0, 2)]
         view.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,
                            gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_DEFAULT)
+        view.drag_source_set(gtk.gdk.BUTTON1_MASK, targets[:2],
+                             gtk.gdk.ACTION_COPY)
         view.connect('drag-data-received', self.__drag_data_received, library)
+        view.connect('drag-data-get', self.__drag_data_get)
         view.connect('drag-motion', self.__drag_motion)
         view.connect('drag-leave', self.__drag_leave)
         if player: view.connect('row-activated', self.__play, player)
@@ -468,6 +470,17 @@ class Playlists(gtk.VBox, Browser):
                     _("Quod Libet can only import playlists in the M3U "
                       "and PLS formats.")).run()
 
+    def __drag_data_get(self, view, ctx, sel, tid, etime):
+        model, iters = self.__view.get_selection().get_selected_rows()
+        songs = []
+        for iter in filter(lambda i: i, iters):
+            songs += list(model[iter][0])
+        if tid == 0:
+            filenames = [song("~filename") for song in songs]
+            sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
+        else:
+            sel.set_uris([song("~uri") for song in songs])
+
     def __select_playlist(self, playlist):
         view = self.__view
         model = view.get_model()
@@ -481,7 +494,8 @@ class Playlists(gtk.VBox, Browser):
             return
         songs = list(model[iter][0])
         songs = filter(lambda s: isinstance(s, AudioFile), songs)
-        menu = SongsMenu(library, songs, playlists=False, remove=False)
+        menu = SongsMenu(
+            library, songs, playlists=False, remove=False, parent=self)
         menu.preseparate()
 
         rem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
