@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: config.py 026dd223d023 2008/09/07 08:23:08 $
 
 # Simple proxy to a Python ConfigParser.
 
@@ -46,13 +44,16 @@ def init(*rc_files):
         "player":
         { "time_remaining": "false",
           "replaygain": "on",
+          "fallback_gain" : "0.0",
+          "pre_amp_gain" : "0.0",
           "backend": "gstbe",
           "gst_pipeline": "",
           },
 
         "library":
         { "exclude": "",
-         },
+          "refresh_on_start": "true",
+          },
 
         # State about the player, to restore on startup
         "memory":
@@ -60,6 +61,7 @@ def init(*rc_files):
           "exfalso_size": "700 500", # ex falso window size
           "browser_size": "500 300", # library browser window size
           "song": "", # filename of last song
+          "seek": "0", # last song position, in milliseconds
           "volume": "1.0", # internal volume, [0.0, 1.0]
           "browser": "SearchBar", # browser name
           "songlist": "true", # on or off
@@ -72,19 +74,22 @@ def init(*rc_files):
         "browsers":
         { "query_text": "", # none/search bar text
           "color": "true", # color search terms in search bar
-          "panes": "~people album", # panes in paned browser
+          "panes": "~people	<~year|<~year> - <album>|<album>>", # panes in paned browser
           "pane_selection": "", # selected pane values
           "background": "", # "global" filter for SearchBar
           "albums": "", # album list
           "album_sort": "0", # album sorting mode, default is title
           "album_covers": "1", # album cover display, on/off
+          "album_substrings": "1", # include substrings in inline search
+          "rating_click": "true", # click to rate song, on/off
+          "rating_confirm_multiple": "false", # confirm rating multiple songs
           },
 
         # Kind of a dumping ground right now, should probably be
         # cleaned out later.
         "settings":
-        { "scan": "", # scan directories, :-separated
-
+        { "scan": "", # scan directories, :-separatd
+          "round": "true", # use rounded corners for artwork thumbnails
           "jump": "true", # scroll song list on current song change
           "default_rating": "0.5", # initial rating of new song
           "ratings": "4", # maximum rating value
@@ -99,7 +104,6 @@ def init(*rc_files):
           "disable_hints": "false",
 
           # search as soon as text is typed into search box
-          # for now this only affects the paned browser
           "eager_search": "true",
           },
 
@@ -119,7 +123,7 @@ def init(*rc_files):
         "plugins": { },
 
         "editing":
-        { "split_on": "& , /", # words to split on
+        { "split_on": "/ & ,", # words to split on
           "id3encoding": "", # ID3 encodings to try
           "save_to_songs": "true",
           "save_email": const.EMAIL,
@@ -127,6 +131,10 @@ def init(*rc_files):
           },
         }
 
+    if os.name == "nt":
+        # special case, to avoid wonderful bursts of static on playback
+        initial["player"]["gst_pipeline"] = "directsoundsink"
+
     for section, values in initial.iteritems():
         _config.add_section(section)
         for key, value in values.iteritems():
