@@ -3,16 +3,18 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: __init__.py c619e6e5936e 2007/11/28 04:23:35 $
 
+import os
 import sys
-import traceback
 
 from glob import glob
 from os.path import dirname, basename, join
+from quodlibet import util, const
 
 base = dirname(__file__)
+if os.name == 'nt' and 'library.zip' in base:
+    # running a py2exe version, use the alternate formats location
+    base = os.path.join(const.BASEDIR, 'formats')
 self = basename(base)
 parent = basename(dirname(base))
 modules = [f[:-3] for f in glob(join(base, "[!_]*.py"))]
@@ -22,7 +24,7 @@ _infos = {}
 for i, name in enumerate(modules):
     try: format = __import__(name, {}, {}, self)
     except Exception, err:
-        traceback.print_exc()
+        util.print_exc()
         continue
     format = __import__(name, {}, {}, self)
     for ext in format.extensions:
@@ -55,21 +57,28 @@ def MusicFile(filename):
                 return _infos[ext](filename)
             except:
                 print_w(_("Error loading %r") % filename)
-                traceback.print_exc()
+                util.print_exc()
                 lt, lv, tb = sys.exc_info()
                 sys.last_type, sys.last_value, sys.last_traceback = lt, lv, tb
                 return None
     else: return None
 
-def supported(song):
-    for ext in _infos.keys():
-        if song.key.lower().endswith(ext):
-            return True
-    return False
+if sys.version_info < (2, 5):
+    def supported(song):
+        lower = song.key.lower()
+        for ext in _infos.keys():
+            if lower.endswith(ext):
+                return True
+        return False
+else:
+    extensions = tuple(_infos.keys())
+    def supported(song):
+        return song.key.lower().endswith(extensions)
 
 def filter(filename):
+    lower = filename.lower()
     for ext in _infos.keys():
-        if filename.lower().endswith(ext): return True
+        if lower.endswith(ext): return True
     return False
 
 from quodlibet.formats._audio import USEFUL_TAGS, MACHINE_TAGS, PEOPLE
