@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: mp4.py d3c5ed7ebabd 2008/04/20 03:37:46 $
 
 import tempfile
 
@@ -35,6 +33,12 @@ class MP4File(AudioFile):
         "\xa9gen": "genre",
         "tmpo": "bpm",
         "\xa9too": "encodedby",
+        "cprt": "copyright",
+        "soal": "albumsort",
+        "soaa": "albumartistsort",
+        "soar": "artistsort",
+        "sonm": "titlesort",
+        "soco": "composersort",
         "----:com.apple.iTunes:MusicBrainz Artist Id":
             "musicbrainz_artistid",
         "----:com.apple.iTunes:MusicBrainz Track Id": "musicbrainz_trackid",
@@ -61,7 +65,7 @@ class MP4File(AudioFile):
         self.__covers = []
         audio = MP4(filename)
         self["~#length"] = int(audio.info.length)
-        self["~#bitrate"] = int(audio.info.bitrate)
+        self["~#bitrate"] = int(audio.info.bitrate / 1000)
         for key, values in audio.items():
             if key in self.__tupletranslate:
                 name = self.__tupletranslate[key]
@@ -85,7 +89,7 @@ class MP4File(AudioFile):
 
     def write(self):
         audio = MP4(self["~filename"])
-        for key in self.__rtranslate.keys() + self.__rtupletranslate.keys():
+        for key in self.__translate.keys() + self.__tupletranslate.keys():
             try: del(audio[key])
             except KeyError: pass
 
