@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: mpc.py 7b30f17a437f 2007/06/27 16:58:32 $
 
 from quodlibet.formats._apev2 import APEv2File
 
@@ -21,7 +19,7 @@ class MPCFile(APEv2File):
         audio = Musepack(filename)
         super(MPCFile, self).__init__(filename, audio)
         self["~#length"] = int(audio.info.length)
-        self["~#bitrate"] = int(audio.info.bitrate)
+        self["~#bitrate"] = int(audio.info.bitrate / 1000)
 
         try:
             if audio.info.title_gain:
