@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: wma.py 8d9e36813f86 2007/11/05 03:20:44 $
 
 from quodlibet.formats._audio import AudioFile
 
@@ -18,6 +16,9 @@ class WMAFile(AudioFile):
     multiple_values = False
     format = "Windows Media Audio"
 
+    #http://msdn.microsoft.com/en-us/library/dd743066%28VS.85%29.aspx
+    #http://msdn.microsoft.com/en-us/library/dd743063%28VS.85%29.aspx
+    #http://msdn.microsoft.com/en-us/library/dd743220%28VS.85%29.aspx
     __translate = {
         "WM/AlbumTitle": "album",
         "Title": "title",
@@ -33,7 +34,7 @@ class WMAFile(AudioFile):
         "WM/TrackNumber": "tracknumber",
         "WM/PartOfSet": "discnumber",
         "WM/BeatsPerMinute": "bpm",
-        "WM/Copyright": "copyright",
+        "Copyright": "copyright",
         "WM/ISRC": "isrc",
         "WM/Mood": "mood",
         "WM/EncodedBy": "encodedby",
@@ -44,6 +45,14 @@ class WMAFile(AudioFile):
         "MusicBrainz/TRM Id": "musicbrainz_trmid",
         "MusicIP/PUID": "musicip_puid",
         "WM/Year": "date",
+        "WM/OriginalArtist": "originalartist",
+        "WM/OriginalAlbumTitle": "originalalbum",
+        "WM/AlbumSortOrder": "albumsort",
+        "WM/ArtistSortOrder": "artistsort",
+        "WM/Genre": "genre",
+        "WM/Publisher": "publisher",
+        "WM/AuthorURL": "website",
+        "Description": "comment"
     }
     __rtranslate = dict([(v, k) for k, v in __translate.iteritems()])
 
@@ -51,7 +60,7 @@ class WMAFile(AudioFile):
         if audio is None:
             audio = mutagen.asf.ASF(filename)
         self["~#length"] = int(audio.info.length)
-        self["~#bitrate"] = int(audio.info.bitrate)
+        self["~#bitrate"] = int(audio.info.bitrate / 1000)
         for name, values in audio.tags.items():
             try: name = self.__translate[name]
             except KeyError: continue
@@ -60,6 +69,10 @@ class WMAFile(AudioFile):
 
     def write(self):
         audio = mutagen.asf.ASF(self["~filename"])
+        for key in self.__translate.keys():
+            try: del(audio[key])
+            except KeyError: pass
+
         for key in self.realkeys():
             try: name = self.__rtranslate[key]
             except KeyError: continue
