@@ -3,8 +3,8 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: xiph.py 262cb6344bcf 2008/11/30 19:24:07 $
+
+import tempfile
 
 import mutagen
 
@@ -23,7 +23,7 @@ class MutagenVCFile(AudioFile):
         if audio is None:
             audio = self.MutagenType(filename)
         self["~#length"] = int(audio.info.length)
-        try: self["~#bitrate"] = int(audio.info.bitrate)
+        try: self["~#bitrate"] = int(audio.info.bitrate / 1000)
         except AttributeError: pass
         for key, value in (audio.tags or {}).items():
             self[key] = "\n".join(value)
@@ -96,18 +96,28 @@ except ImportError: OggVorbis = None
 else:
     extensions.append(".ogg")
     extensions.append(".oga")
+    ogg_formats.append(OggVorbis)
 
-try: from mutagen.flac import FLAC
+try: from mutagen.flac import FLAC, FLACNoHeaderError
 except ImportError: FLAC = None
-else: extensions.append(".flac")
+else:
+    extensions.append(".flac")
+    ogg_formats.append(FLAC)
 
 try: from mutagen.oggflac import OggFLAC
 except ImportError: OggFLAC = None
-else: extensions.append(".oggflac")
+else:
+    extensions.append(".oggflac")
+    ogg_formats.append(OggFLAC)
 
 try: from mutagen.oggspeex import OggSpeex
 except ImportError: OggSpeex = None
-else: extensions.append(".spx")
+else:
+    extensions.append(".spx")
+    ogg_formats.append(OggSpeex)
+
+try: from mutagen.id3 import ID3
+except ImportError: ID3 = None
 
 class OggFile(MutagenVCFile):
     format = "Ogg Vorbis"
@@ -125,16 +135,53 @@ class FLACFile(MutagenVCFile):
     format = "FLAC"
     MutagenType = FLAC
 
+    def __init__(self, filename, audio=None):
+        if audio is None:
+            audio = FLAC(filename)
+        super(FLACFile, self).__init__(filename, audio)
+        if audio.pictures:
+            self["~picture"] = "y"
+
+    def get_format_cover(self):
+        try:
+            tag = FLAC(self["~filename"])
+        except EnvironmentError:
+            return None
+        else:
+            covers = tag.pictures
+            if not covers:
+                return None
+
+            for cover in covers:
+                if cover.type == 3:
+                    pic = cover
+                    break
+            else:
+                pic = covers[0]
+
+            fn = tempfile.NamedTemporaryFile()
+            fn.write(pic.data)
+            fn.flush()
+            fn.seek(0, 0)
+            return fn
+
+    def write(self):
+        if ID3 is not None:
+            ID3().delete(filename=self["~filename"])
+        super(FLACFile, self).write()
+
 def info(filename):
-    try: audio = mutagen.File(filename)
+    try: audio = mutagen.File(filename, options = ogg_formats)
     except AttributeError:
         audio = OggVorbis(filename)
+    if audio is None and FLAC is not None:
+        # FLAC with ID3
+        try: audio = FLAC(filename)
+        except FLACNoHeaderError: pass
     if audio is None:
         raise IOError("file type could not be determined")
-    else:
-        Kind = type(audio)
-        for klass in globals().values():
-            if Kind is getattr(klass, 'MutagenType', None):
-                return klass(filename, audio)
-        else:
-            raise IOError("file type could not be determined")
+    Kind = type(audio)
+    for klass in globals().values():
+        if Kind is getattr(klass, 'MutagenType', None):
+            return klass(filename, audio)
+    raise IOError("file type could not be determined")
