@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: songs.py b156fbbab40a 2008/11/30 19:53:05 $
 
 """Song library classes.
 
@@ -13,7 +11,6 @@ close enough.
 """
 
 import os
-import traceback
 
 from quodlibet import util
 
@@ -151,7 +148,7 @@ class FileLibrary(Library):
                     item.reload()
                 except StandardError:
                     print_d("Error reloading %r." % item.key, self)
-                    traceback.print_exc()
+                    util.print_exc()
                     return False, True
                 else:
                     print_d("Reloaded %r." % item.key, self)
@@ -271,7 +268,7 @@ class FileLibrary(Library):
             fullpath = os.path.expanduser(fullpath)
             if filter(fullpath.startswith, exclude):
                 continue
-            for path, dnames, fnames in os.walk(fullpath):
+            for path, dnames, fnames in os.walk(util.fsnative(fullpath)):
                 for filename in fnames:
                     fullfilename = os.path.join(path, filename)
                     if filter(fullfilename.startswith, exclude):
@@ -350,4 +347,4 @@ class SongFileLibrary(SongLibrary, FileLibrary):
             else:
                 return self._contents[filename]
         except StandardError:
-            traceback.print_exc()
+            util.print_exc()
