@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: _pattern.py bf637a65be70 2008/09/07 22:38:36 $
 
 # Pattern := (<String> | <Tags>)*
 # String := ([^<>|\\]|\\.)+, a string
@@ -161,6 +159,17 @@ class _Pattern(PatternParser):
         p = PatternParser(self.__tokens)
         return self._post(p.Pattern(self.Song(song, self._formatters)), song)
 
+    def real_tags(self, cond=True):
+        tags = []
+        tokens = self.__tokens
+        for i, tok in enumerate(tokens):
+            if tok.type == TEXT and \
+                tokens[max(0, i-1)].type == OPEN and \
+                tokens[i+1].type != EOF and \
+                (tokens[i+1].type != COND or cond):
+                tags.extend(util.tagsplit(tok.lexeme))
+        return [t for i, t in enumerate(tags) if i == tags.index(t)]
+
     def _post(self, value, song): return value
 
     __mod__ = format
@@ -185,13 +194,22 @@ class _FileFromPattern(_Pattern):
                    (lambda k, s: (len(s) > 100 and s[:100] + "...") or s),
                    ]
 
+    def __init__(self, string):
+        # On Windows, users may use backslashes in patterns as path separators.
+        # Since Windows filenames can't use '<>|' anyway, preserving backslash
+        # escapes is unnecessary, so we just replace them blindly.
+        if os.name == 'nt':
+            string = string.replace("\\", "/")
+        super(_FileFromPattern, self).__init__(string)
+
     def _post(self, value, song):
         if value:
             fn = song.get("~filename", ".")
-            ext = fn[fn.rfind("."):]
-            if not value.endswith(ext): value += ext
+            ext = fn[fn.rfind("."):].lower()
+            val_ext = value[-len(ext):].lower()
+            if not ext == val_ext: value += ext.lower()
             value = os.path.expanduser(value)
-            if "/" in value and not value.startswith("/"):
+            if "/" in value and not os.path.isabs(value):
                 raise ValueError("Pattern is not rooted")
         return value
 
