@@ -64,10 +64,12 @@ class BasePlayer(gtk.Object):
         self.props.volume = min(1.0, max(0.0, v))
     volume = property(lambda s: s._volume, _set_volume)
 
-    def setup(self, source, song):
+    def setup(self, source, song, seek_pos):
         """Connect to a PlaylistModel, and load a song."""
         self._source = source
         self.go_to(song)
+        if seek_pos:
+            self.seek(seek_pos)
 
     def remove(self, song):
         if self.song is song:
@@ -93,7 +95,7 @@ class BasePlayer(gtk.Object):
     def previous(self):
         # Go back if standing at the beginning of the song,
         # otherwise restart the current song.
-        if self.get_position() < 500:
+        if self.get_position() < 1000:
             self._source.previous()
             self._end(True)
         else:
