@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: xinebe.py d112be42fd69 2008/09/14 00:10:29 $
 
 import gobject
 import time
@@ -115,12 +113,14 @@ class XinePlaylistPlayer(BasePlayer):
     def do_set_property(self, property, v):
         if property.name == 'volume':
             self._volume = v
-            if self.song is not None:
-                if config.getboolean("player", "replaygain"):
-                    profiles = filter(None, self.replaygain_profiles)[0]
-                    v = max(0.0, min(4.0, v * self.song.replay_gain(profiles)))
+            if self.song and config.getboolean("player", "replaygain"):
+                profiles = filter(None, self.replaygain_profiles)[0]
+                fb_gain = config.getfloat("player", "fallback_gain")
+                pa_gain = config.getfloat("player", "pre_amp_gain")
+                scale = self.song.replay_gain(profiles, pa_gain, fb_gain)
+                v = max(0.0, v * scale)
             v = min(100, int(v * 100))
-            xine_set_param(self._stream, XINE_PARAM_AUDIO_VOLUME, v)
+            xine_set_param(self._stream, XINE_PARAM_AUDIO_AMP_LEVEL, v)
         else:
             raise AttributeError
 
@@ -203,6 +203,12 @@ class XinePlaylistPlayer(BasePlayer):
             self.paused = True
             xine_stop(self._stream)
 
+    def setup(self, playlist, song, seek_pos):
+        super(XinePlaylistPlayer, self).setup(playlist, song, seek_pos)
+        # xine's declining to seek so soon after startup; try again in 100ms
+        if seek_pos:
+            gobject.timeout_add(100, self.seek, seek_pos)
+
 _xine = None
 _plugins = None
 
