@@ -4,13 +4,10 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: events.py 86de5f1fd15f 2008/09/07 07:34:58 $
-
-import traceback
 
 import gobject
 
+from quodlibet import util
 from quodlibet.plugins import Manager, SongWrapper, ListWrapper
 
 class EventPlugin(object):
@@ -100,7 +97,7 @@ class EventPlugins(Manager):
 
             try: obj = Kind()
             except:
-                traceback.print_exc()
+                util.print_exc()
             else:
                 if obj.PLUGIN_ID in self._plugins:
                     self._plugins[obj.PLUGIN_ID].destroy()
@@ -116,7 +113,7 @@ class EventPlugins(Manager):
                 if enabled: plugin.enabled()
                 else: plugin.disabled()
             except:
-                traceback.print_exc()
+                util.print_exc()
 
     def __invoke(self, librarian, event, *args):
         try:
@@ -133,7 +130,7 @@ class EventPlugins(Manager):
                 if handler is not None:
                     try: handler(*args)
                     except Exception:
-                        traceback.print_exc()
+                        util.print_exc()
         finally:
             if event not in ["removed", "changed"] and args:
                 from quodlibet.widgets import main
