@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: browser.py 1d1a43280369 2007/05/18 07:44:07 $
 
 import gtk
 import pango
@@ -14,6 +12,7 @@ from quodlibet import util
 
 from quodlibet.qltk.songlist import SongList
 from quodlibet.qltk.x import Window
+from quodlibet.qltk.x import RPaned
 
 class LibraryBrowser(Window):
     def __init__(self, Kind, library):
@@ -26,6 +25,9 @@ class LibraryBrowser(Window):
         try: x, y = map(int, config.get('memory', cfg_name).split())
         except (config.error, ValueError):
             x, y = 500, 300
+        screen = self.get_screen()
+        x = min(x, screen.get_width())
+        y = min(y, screen.get_height())
         self.set_default_size(x, y)
         self.connect('configure-event', LibraryBrowser.__save_size, cfg_name)
 
@@ -46,7 +48,8 @@ class LibraryBrowser(Window):
         if browser.accelerators:
             self.add_accel_group(browser.accelerators)
 
-        self.child.pack_start(browser.pack(sw))
+        self.__container = browser.pack(sw)
+        self.child.pack_start(self.__container)
 
         self.__statusbar = gtk.Label()
         self.__statusbar.set_text(_("No time information"))
@@ -55,6 +58,7 @@ class LibraryBrowser(Window):
         self.child.pack_end(self.__statusbar, expand=False)
 
         browser.connect('songs-selected', self.__browser_cb)
+        browser.finalize(False)
         view.connect('popup-menu', self.__menu, library)
         view.connect('drag-data-received', self.__drag_data_recv)
         view.connect('row-activated', self.__enqueue)
@@ -67,6 +71,16 @@ class LibraryBrowser(Window):
             c.show()
         self.child.show()
         self.show()
+        self.__set_pane_size()
+
+    def __set_pane_size(self):
+        if isinstance(self.__container, RPaned):
+            try:
+                key = "%s_pos" % self.browser.__class__.__name__
+                val = config.getfloat("browsers", key)
+            except:
+                val = 0.4
+            self.__container.set_relative(val)
 
     def __browser_cb(self, browser, songs, sorted):
         self.__set_time(songs=songs)
