@@ -1,10 +1,8 @@
-# Copyright 2005 Joe Wreschnig, Michael Urman
+# Copyright 2005-2010 Joe Wreschnig, Michael Urman, Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: cbes.py ff27fdba924a 2007/10/13 23:57:08 $
 
 import os
 
@@ -14,6 +12,7 @@ import pango
 from quodlibet import qltk
 
 from quodlibet.qltk.views import RCMHintedTreeView
+from quodlibet.qltk import ClearButton
 
 class CBESEditor(qltk.Window):
     def __init__(self, cbes):
@@ -184,6 +183,15 @@ class ComboBoxEntrySave(gtk.ComboBoxEntry):
         self.connect_object('destroy', self.set_model, None)
         self.connect_object('changed', self.__changed, model)
 
+    def pack_clear_button(self, container=None):
+        if getattr(self.child, "set_icon_from_stock", None):
+            self.child.set_icon_from_stock(
+                gtk.ENTRY_ICON_SECONDARY, gtk.STOCK_CLEAR)
+            clear = lambda *x: x[0].set_text("")
+            self.child.connect("icon-release", clear)
+        else:
+            container.pack_start(ClearButton(self.child), False)
+
     def __changed(self, model):
         iter = self.get_active_iter()
         if iter:
@@ -191,8 +199,14 @@ class ComboBoxEntrySave(gtk.ComboBoxEntry):
                 Kind = ICONS[model[iter][2]]
                 Kind(self)
                 self.set_active(-1)
+            else:
+                self.__focus_entry()
         model[self.__special_iter][0] = self.child.get_text()
 
+    def __focus_entry(self):
+        self.child.grab_focus()
+        self.child.emit('move-cursor', gtk.MOVEMENT_BUFFER_ENDS, 0, False)
+
     def __fill(self, filename, initial):
         model = self.get_model()
         self.__special_iter = model.append(
