@@ -3,12 +3,10 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: completion.py 1e91b6115576 2008/09/07 02:51:17 $
 
 import gtk
 
-from quodlibet import formats
+from quodlibet import formats, config
 
 from quodlibet.util import copool
 
@@ -85,9 +83,11 @@ class LibraryTagCompletion(EntryWordCompletion):
     @classmethod
     def __update_song(klass, library, songs, model):
         print_d("Updating tag model for %d songs" % len(songs))
+        tags = klass.__tags
         for song in songs:
             for tag in song.keys():
-                if not (tag.startswith("~#") or tag in formats.MACHINE_TAGS):
+                if not (tag.startswith("~#") or tag in formats.MACHINE_TAGS
+                        or tag in tags):
                     klass.__tags.add(tag)
                     model.append([tag])
         print_d("Done updating tag model for %d songs" % len(songs))
@@ -121,7 +121,9 @@ class LibraryValueCompletion(gtk.EntryCompletion):
         self.set_tag(tag, library)
 
     def set_tag(self, tag, library):
-        if tag is None:
+        if not config.getboolean("settings", "eager_search"):
+            return
+        elif tag is None:
             return
         elif tag in ("bpm date discnumber isrc originaldate recordingdate "
                      "tracknumber title").split() + formats.MACHINE_TAGS:
