@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: controls.py 1d1a43280369 2007/05/18 07:44:07 $
 
 import gobject
 import gtk
@@ -139,7 +137,7 @@ class SeekBar(HSlider):
 
 class Volume(VSlider):
     def __init__(self, device):
-        i = gtk.image_new_from_stock(stock.VOLUME_MAX, SIZE)
+        i = gtk.Image()
         super(type(self), self).__init__(i)
         self.scale.set_update_policy(gtk.UPDATE_CONTINUOUS)
         self.scale.set_inverted(True)
@@ -166,7 +164,11 @@ class Volume(VSlider):
         elif val < 0.33: img = stock.VOLUME_MIN
         elif val < 0.66: img = stock.VOLUME_MED
         else: img = stock.VOLUME_MAX
-        image.set_from_stock(img, SIZE)
+
+        if gtk.icon_theme_get_default().has_icon(img):
+            image.set_from_icon_name(img, SIZE)
+        else:
+            image.set_from_stock(img, SIZE)
 
         device.volume = val
         config.set("memory", "volume", str(slider.get_value()))
@@ -175,13 +177,22 @@ class Volume(VSlider):
         self.scale.set_value(device.props.volume)
 
 class StopAfterMenu(gtk.Menu):
+    __menu = None
+
+    def __new__(klass, parent):
+        if klass.__menu is None:
+            return super(StopAfterMenu, klass).__new__(klass)
+        else: return klass.__menu
+
     def __init__(self, player):
+        if type(self).__menu: return
+        else: type(self).__menu = self
         super(StopAfterMenu, self).__init__()
         self.__item = gtk.CheckMenuItem(_("Stop after this song"))
         self.__item.set_active(False)
         self.append(self.__item)
         player.connect('paused', self.__paused)
-        player.connect('song-ended', self.__ended)
+        player.connect_after('song-ended', self.__ended)
         self.__item.show()
 
     def __paused(self, player):
@@ -198,37 +209,39 @@ class StopAfterMenu(gtk.Menu):
         return self.__item.set_active(active)
     active = property(__get_active, __set_active)
 
-class PlayControls(gtk.VBox):
+class PlayControls(gtk.Table):
     def __init__(self, player, library):
-        gtk.VBox.__init__(self, spacing=3)
-        hbox = gtk.HBox(spacing=3)
+        gtk.Table.__init__(self, rows=2, columns=2, homogeneous=True)
+
+        self.set_row_spacings(3)
+        self.set_col_spacings(3)
+
         prev = gtk.Button()
         prev.add(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PREVIOUS, SIZE))
-        hbox.pack_start(prev)
+        self.attach(prev, 0, 1, 0, 1, yoptions=gtk.FILL)
 
         play = gtk.ToggleButton()
         play.add(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, SIZE))
-        hbox.pack_start(play)
+        self.attach(play, 1, 2, 0, 1, yoptions=gtk.FILL)
+
         safter = StopAfterMenu(player)
 
         next = gtk.Button()
         next.add(gtk.image_new_from_stock(gtk.STOCK_MEDIA_NEXT, SIZE))
-        hbox.pack_start(next)
+        self.attach(next, 2, 3, 0, 1, yoptions=gtk.FILL)
 
-        self.pack_start(hbox, expand=False, fill=False)
-
-        hbox = gtk.HBox(spacing=3)
         self.volume = Volume(player)
-        hbox.pack_start(self.volume, expand=False)
-        hbox.pack_start(SeekBar(player, library))
-        self.pack_start(hbox, expand=False, fill=False)
+        self.attach(self.volume, 0, 1, 1, 2, yoptions=gtk.FILL)
+
+        seekbar = SeekBar(player, library)
+        self.attach(seekbar, 1, 3, 1, 2, yoptions=gtk.FILL)
 
         prev.connect_object('clicked', self.__previous, player)
         play.connect('toggled', self.__playpause, player)
         play.connect('button-press-event', self.__play_button_press, safter)
         play.connect_object('popup-menu', self.__popup, safter, play.child)
         next.connect_object('clicked', self.__next, player)
-        player.connect('song-started', self.__song_started, next)
+        player.connect('song-started', self.__song_started, next, play)
         player.connect_object('paused', play.set_active, False)
         player.connect_object('unpaused', play.set_active, True)
         self.show_all()
@@ -245,7 +258,7 @@ class PlayControls(gtk.VBox):
             safter.popup(None, None, None, button, time)
         return True
 
-    def __song_started(self, player, song, next):
+    def __song_started(self, player, song, next, play):
         next.set_sensitive(bool(song))
 
     def __playpause(self, button, player):
