@@ -569,9 +569,9 @@ class EditTags(gtk.VBox):
             if row[EDITED] and row[RENAMED] and not row[DELETED]:
                 renamed.setdefault(row[TAG], [])
                 renamed[row[TAG]].append((util.decode(row[ORIGTAG]),
-                                          util.decode(row[ORIGVALUE]),
-                                          util.decode(row[VALUE])))
-
+                                          util.decode(row[VALUE]),
+                                          util.decode(row[ORIGVALUE])))
+                
         was_changed = []
         songs = self.__songinfo.songs
         win = WritingWindow(self, len(songs))
@@ -608,16 +608,16 @@ class EditTags(gtk.VBox):
                         changed = True
             save_rename = []
             for new_tag, values in renamed.iteritems():
-                for old_tag, old_value, new_value in values:
+                for old_tag, new_value, old_value in values:
                     old_tag = util.unescape(old_tag)
                     old_value = util.unescape(old_value)
                     new_value = util.unescape(new_value)
                     if (song.can_change(new_tag) and
                         song.can_change(old_tag) and old_tag in song):
-                        if not is_special(old_value):
+                        if not is_special(new_value):
                             song.remove(old_tag, old_value)
                             save_rename.append((new_tag, new_value))
-                        elif is_missing(old_value):
+                        elif is_missing(new_value):
                             value = strip_missing(old_value)
                             song.remove(old_tag, old_value)
                             save_rename.append((new_tag, new_value))
