@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: exfalsowindow.py ff27fdba924a 2007/10/13 23:57:08 $
 
 import os
 
@@ -18,6 +16,7 @@ from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
 from quodlibet import stock
+from quodlibet import util
 
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.delete import DeleteDialog
@@ -76,13 +75,20 @@ class ExFalsoWindow(gtk.Window):
         bbox.pack_start(l)
         
         if os.name == "nt":
-            folders = [os.path.join(const.HOME, "My Documents")]
-            for letter in "CDEFGHIJKLMNOPQRSTUVWXYZ":
-                # Skip the floppy drives to avoid waiting
-                if os.path.isdir(letter + ":\\"):
-                    folders.append(letter + ":\\")
+            # use HOME, Documents (if not under HOME), and non-floppy drives
+            folders = [const.HOME]
+            try: from win32com.shell import shell, shellcon as con
+            except ImportError, e: pass
+            else:
+                documents = shell.SHGetFolderPath(0, con.CSIDL_PERSONAL, 0, 0)
+                if not documents.startswith(const.HOME + os.sep):
+                    folders.append(documents)
+                music = shell.SHGetFolderPath(0, con.CSIDL_MYMUSIC, 0, 0)
+                folders.insert(0, music)
+            folders = filter(os.path.isdir, folders +
+                [letter + ":\\" for letter in "CDEFGHIJKLMNOPQRSTUVWXYZ"])
         else:
-            folders=[const.HOME, "/"]
+            folders = [const.HOME, "/"]
 
         fs = FileSelector(dir, folders=folders)
 
@@ -103,7 +109,9 @@ class ExFalsoWindow(gtk.Window):
         self.__save = None
         self.connect_object('changed', self.set_pending, None)
         for c in fs.get_children():
-            c.child.connect('button-press-event', self.__pre_selection_changed)
+            c.child.connect('button-press-event',
+                self.__pre_selection_changed, fs, nb)
+            c.child.connect('focus', self.__pre_selection_changed, fs, nb)
         fs.get_children()[1].child.connect('popup-menu', self.__popup_menu, fs)
         self.emit('changed', [])
 
@@ -114,7 +122,11 @@ class ExFalsoWindow(gtk.Window):
         self.__ag.connect_group(key, mod, 0, gtk.main_quit)
         self.add_accel_group(self.__ag)
         self.connect('configure-event', ExFalsoWindow.__save_size)
-        self.resize(*map(int, config.get("memory", "exfalso_size").split()))
+        x, y = map(int, config.get("memory", "exfalso_size").split())
+        screen = self.get_screen()
+        x = min(x, screen.get_width())
+        y = min(y, screen.get_height())
+        self.resize(x, y)
 
     def __save_size(self, event):
         config.set("memory", "exfalso_size",
@@ -123,12 +135,14 @@ class ExFalsoWindow(gtk.Window):
     def set_pending(self, button, *excess):
         self.__save = button
 
-    def __pre_selection_changed(self, view, event):
+    def __pre_selection_changed(self, view, event, fs, nb):
         if self.__save:
             resp = qltk.CancelRevertSave(self).run()
             if resp == gtk.RESPONSE_YES: self.__save.clicked()
-            elif resp == gtk.RESPONSE_NO: return False
-            else: return True # cancel or closed
+            elif resp == gtk.RESPONSE_NO: FileSelector.rescan(fs)
+            else:
+                nb.grab_focus()
+                return True # cancel or closed
 
     def __popup_menu(self, view, fs):
         view.grab_focus()
@@ -166,11 +180,15 @@ class ExFalsoWindow(gtk.Window):
         label.set_text(ngettext("%d song", "%d songs", count) % count)
 
         for row in rows:
-            filename = model[row][0]
+            filename = util.fsnative(model[row][0])
             if not os.path.exists(filename): pass
             elif filename in self.__library:
-                files.append(self.__library[filename])
-            else: files.append(formats.MusicFile(model[row][0]))
+                file = self.__library[filename]
+                if file("~#mtime") + 1. < util.mtime(filename):
+                    try: file.reload()
+                    except StandardError: pass
+                files.append(file)
+            else: files.append(formats.MusicFile(filename))
         files = filter(None, files)
         if len(files) == 0: self.set_title("Ex Falso")
         elif len(files) == 1:
@@ -185,17 +203,9 @@ class ExFalsoWindow(gtk.Window):
         self.__library.add(files)
         self.emit('changed', files)
 
-class PreferencesWindow(qltk.Window):
-    __window = None
-
-    def __new__(klass, parent):
-        if klass.__window is None:
-            return super(PreferencesWindow, klass).__new__(klass)
-        else: return klass.__window
-
+class PreferencesWindow(qltk.UniqueWindow):
     def __init__(self, parent):
-        if type(self).__window: return
-        else: type(self).__window = self
+        if self.is_not_unique(): return
         super(PreferencesWindow, self).__init__()
         self.set_title(_("Ex Falso Preferences"))
         self.set_border_width(12)
@@ -227,5 +237,4 @@ class PreferencesWindow(qltk.Window):
         config.set(section, name, entry.get_text())
 
     def __destroy(self):
-        type(self).__window = None
         config.write(const.CONFIG)
