@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: info.py 3673eded203f 2008/09/07 04:42:10 $
 
 import os
 
@@ -62,13 +60,13 @@ class SongInfo(gtk.Label):
         sep.show()
         props = gtk.ImageMenuItem(stock.EDIT_TAGS)
         props.connect_object(
-            'activate', SongProperties, library, [player.song])
+            'activate', SongProperties, library, [player.song], self)
         props.show()
         props.set_sensitive(bool(player.song))
         menu.append(props)
         info = gtk.ImageMenuItem(gtk.STOCK_INFO)
         info.connect_object(
-            'activate', Information, library, [player.song])
+            'activate', Information, library, [player.song], self)
         info.show()
         menu.append(info)
         info.set_sensitive(bool(player.song))
