@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: information.py 0a66840a1115 2008/11/30 19:57:32 $
 
 import time
 
@@ -267,7 +265,7 @@ class OneSong(qltk.Notebook):
         size = util.format_size(util.size(song["~filename"]))
         mtime = ftime(util.mtime(song["~filename"]))
         if "~#bitrate" in song and song["~#bitrate"] != 0:
-            bitrate = _("%d kbps") % int(song["~#bitrate"]/1000)
+            bitrate = _("%d kbps") % int(song["~#bitrate"])
         else: bitrate = False
 
         t = gtk.Table(4, 2)
@@ -467,7 +465,6 @@ class OneArtist(qltk.Notebook):
         l.set_ellipsize(pango.ELLIPSIZE_END)
         box.pack_start(Frame(_("Selected Discography"), l))
 
-        tips = qltk.Tooltips(self)
         covers = [ac for ac in covers if bool(ac[1])]
         t = gtk.Table(4, (len(covers) // 4) + 1)
         t.set_col_spacings(12)
@@ -476,7 +473,7 @@ class OneArtist(qltk.Notebook):
         for i, (album, cover, song) in enumerate(covers):
             if cover.name in added: continue
             cov = CoverImage([70, 70], song)
-            tips.set_tip(cov.child, album)
+            cov.child.set_tooltip_text(album)
             c = i % 4
             r = i // 4
             t.attach(cov, c, c + 1, r, r + 1,
@@ -557,7 +554,7 @@ class ManySongs(qltk.Notebook):
                        expand=False, fill=False)
 
 class Information(Window):
-    def __init__(self, library, songs):
+    def __init__(self, library, songs, parent=None):
         super(Information, self).__init__()
         self.set_border_width(12)
         self.set_default_size(400, 400)
@@ -569,6 +566,7 @@ class Information(Window):
             self.connect_object('destroy', library.disconnect, sig)
         self.__songs = songs
         self.__update(library)
+        self.set_transient_for(qltk.get_top_parent(parent))
         self.show_all()
 
     def __check_changed(self, library, songs):
