@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
+# Copyright 2004-2010 Joe Wreschnig, Michael Urman, Iñigo Serna,
+#                     Steven Robertson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: prefs.py 8d9e36813f86 2007/11/05 03:20:44 $
 
 import os
 import gtk
@@ -22,21 +21,13 @@ from quodlibet.qltk.chooser import FolderChooser
 from quodlibet.qltk.entry import ValidatingEntry
 from quodlibet.qltk.songlist import SongList
 
-class PreferencesWindow(qltk.Window):
-    __window = None
-
-    def __new__(klass, parent):
-        if klass.__window is None:
-            return super(PreferencesWindow, klass).__new__(klass)
-        else: return klass.__window
-
+class PreferencesWindow(qltk.UniqueWindow):
     class SongList(gtk.VBox):
         def __init__(self):
             super(PreferencesWindow.SongList, self).__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Song List")
             vbox = gtk.VBox(spacing=12)
-            tips = qltk.Tooltips(self)
 
             buttons = {}
             table = gtk.Table(3, 3)
@@ -92,8 +83,8 @@ class PreferencesWindow(qltk.Window):
             others = gtk.Entry()
             if "~current" in checks: checks.remove("~current")
             others.set_text(" ".join(checks))
-            tips.set_tip(
-                others, _("Other columns to display, separated by spaces"))
+            others.set_tooltip_text(
+                _("Other columns to display, separated by spaces"))
             l.set_mnemonic_widget(others)
             l.set_use_underline(True)
             hbox.pack_start(others)
@@ -136,13 +127,6 @@ class PreferencesWindow(qltk.Window):
             super(PreferencesWindow.Browsers, self).__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Browsers")
-            tips = qltk.Tooltips(self)
-            c = ConfigCheckButton(
-                _("Color _search terms"), 'browsers', 'color')
-            c.set_active(config.getboolean("browsers", "color"))
-            tips.set_tip(
-                c, _("Display simple searches in blue, "
-                     "advanced ones in green, and invalid ones in red"))
             hb = gtk.HBox(spacing=6)
             l = gtk.Label(_("_Global filter:"))
             l.set_use_underline(True)
@@ -154,9 +138,45 @@ class PreferencesWindow(qltk.Window):
             hb.pack_start(e)
             self.pack_start(hb, expand=False)
 
-            f = qltk.Frame(_("Search Library"), child=c)
+            vb = gtk.VBox(spacing=6)
+            c = ConfigCheckButton(
+                _("Search after _typing"), 'settings', 'eager_search')
+            c.set_active(config.getboolean('settings', 'eager_search'))
+            c.set_tooltip_text(_("Show search results after the user "
+                "stops typing."))
+            vb.pack_start(c)
+            c = ConfigCheckButton(
+                _("Color _search terms"), 'browsers', 'color')
+            c.set_active(config.getboolean("browsers", "color"))
+            c.set_tooltip_text(_("Display simple searches in blue, "
+                     "advanced ones in green, and invalid ones in red"))
+            vb.pack_start(c)
+            f = qltk.Frame(_("Search Library"), child=vb)
             self.pack_start(f, expand=False)
 
+            c1 = ConfigCheckButton(
+                _("Confirm _multiple ratings"), 
+                'browsers', 'rating_confirm_multiple')
+            c1.set_active(
+                config.getboolean("browsers", "rating_confirm_multiple"))
+            c1.set_tooltip_text(_("Ask for confirmation before changin the "
+                     "rating of multiple songs at once"))
+
+            c2 = ConfigCheckButton(
+                _("Enable _one-click ratings"),
+               'browsers', 'rating_click')
+            c2.set_active(
+                config.getboolean("browsers", "rating_click"))
+            c2.set_tooltip_text(_("Enable rating by clicking on the rating  "
+                     "column in the song list"))
+
+            vbox = gtk.VBox(spacing=6)
+            vbox.pack_start(c1, expand=False)
+            vbox.pack_start(c2, expand=False)
+
+            f1 = qltk.Frame(_("Ratings"), child=vbox)
+            self.pack_start(f1, expand=False)          
+
         def _entry(self, entry, name, section="settings"):
             config.set(section, name, entry.get_text())
 
@@ -166,24 +186,95 @@ class PreferencesWindow(qltk.Window):
             self.set_border_width(12)
             self.title = _("Player")
 
-            tips = qltk.Tooltips(self)
+            if config.get('player', 'backend') == "gstbe":
+                e = gtk.Entry()
+                e.set_tooltip_text(_("The GStreamer output pipeline used for "
+                        "playback, such as 'alsasink device=default'. "
+                        "Leave blank for default pipeline."))
+                e.set_text(config.get('player', 'gst_pipeline'))
+                def changed(entry):
+                    config.set('player', 'gst_pipeline', entry.get_text())
+                e.connect('changed', changed)
+                l = gtk.Label(_('_Output pipeline:'))
+                l.set_use_underline(True)
+                l.set_mnemonic_widget(e)
+                b = gtk.Button(stock=gtk.STOCK_APPLY)
+                b.connect('clicked',
+                        lambda w: player.device.go_to(player.device.song))
+
+                hb = gtk.HBox(spacing=6)
+                hb.pack_start(l, expand=False)
+                hb.pack_start(e)
+                hb.pack_start(b, expand=False)
+                self.pack_start(hb, expand=False)
+
             c = ConfigCheckButton(
                 _("_Jump to playing song automatically"), 'settings', 'jump')
-            tips.set_tip(c, _("When the playing song changes, "
-                              "scroll to it in the song list"))
+            c.set_tooltip_text(_("When the playing song changes, "
+                                 "scroll to it in the song list"))
             c.set_active(config.getboolean("settings", "jump"))
             self.pack_start(c, expand=False)
 
             c = ConfigCheckButton(
-                _("_Replay Gain volume adjustment"), "player", "replaygain")
-            c.set_active(config.getboolean("player", "replaygain"))
+                _("_Use rounded corners on thumbnails"), 'settings', 'round')
+            c.set_tooltip_text(_("Round the corners of album artwork "
+                "thumbnail images. May require restart to take effect."))
+            c.set_active(config.getboolean("settings", "round"))
             self.pack_start(c, expand=False)
+
+            vbox = gtk.VBox(spacing=6)
+            c = ConfigCheckButton(_("_Enable Replay Gain volume adjustment"),
+                                    "player", "replaygain")
+            c.set_active(config.getboolean("player", "replaygain"))
             c.connect('toggled', self.__toggled_gain)
+            vbox.pack_start(c, expand=False)
+            try:
+                fallback_gain = config.getfloat("player", "fallback_gain")
+            except:
+                fallback_gain = 0.0
+            adj = gtk.Adjustment(fallback_gain, -12.0, 12.0, 0.5, 0.5, 0.0)
+            s = gtk.SpinButton(adj)
+            s.set_digits(1)
+            s.connect('changed', self.__changed, 'player', 'fallback_gain')
+            s.set_tooltip_text(_("If no Replay Gain information is available "
+                                 "for a song, scale the volume by this value"))
+            l = gtk.Label(_("Fall-back gain (dB):"))
+            l.set_use_underline(True)
+            l.set_mnemonic_widget(s)
+
+            hb = gtk.HBox(spacing=6)
+            hb.pack_start(l, expand=False)
+            hb.pack_start(s)
+            vbox.pack_start(hb, expand=False)
+            try:
+                pre_amp_gain = config.getfloat("player", "pre_amp_gain")
+            except:
+                pre_amp_gain = 0
+            adj = gtk.Adjustment(pre_amp_gain, -6, 6, 0.5, 0.5, 0.0)
+            adj.connect('value-changed', self.__changed,
+                        'player', 'pre_amp_gain')
+            s = gtk.SpinButton(adj)
+            s.set_digits(1)
+            s.set_tooltip_text(_("Scale volume for all songs by this value, "
+                                 "as long as the result will not clip"))
+            l = gtk.Label(_("Pre-amp gain (dB):"))
+            l.set_use_underline(True)
+            l.set_mnemonic_widget(s)
+            hb = gtk.HBox(spacing=6)
+            hb.pack_start(l, expand=False)
+            hb.pack_start(s)
+            vbox.pack_start(hb, expand=False)
+            f = qltk.Frame(_("Replay Gain Volume Adjustment"), child=vbox)
+            self.pack_start(f)
             self.show_all()
 
         def __toggled_gain(self, activator):
             player.playlist.volume = player.playlist.volume
 
+        def __changed(self, adj, section, name):
+            config.set(section, name, str(adj.get_value()))
+            player.playlist.volume = player.playlist.volume
+
     class Library(gtk.VBox):
         def __init__(self):
             super(PreferencesWindow.Library, self).__init__(spacing=12)
@@ -194,12 +285,10 @@ class PreferencesWindow(qltk.Window):
             e = gtk.Entry()
             e.set_text(util.fsdecode(config.get("settings", "scan")))
             hb.pack_start(e)
-            tips = qltk.Tooltips(self)
-            tips.set_tip(
-                e, _("Songs placed in these folders (separated by ':') "
-                     "will be added to your library"))
+            e.set_tooltip_text(_("Songs placed in these folders (separated "
+                     "by ':') will be added to your library"))
             hb.pack_start(b, expand=False)
-            scandirs = config.get("settings", "scan").split(":")
+            scandirs = util.split_scan_dirs(config.get("settings", "scan"))
             if scandirs and os.path.isdir(scandirs[-1]):
                 # start with last added directory
                 initial = scandirs[-1]
@@ -207,7 +296,14 @@ class PreferencesWindow(qltk.Window):
                 initial = const.HOME
             b.connect('clicked', self.__select, e, initial)
             e.connect('changed', self.__changed, 'settings', 'scan')
-            f = qltk.Frame(_("Scan _Directories"), child=hb)
+
+            cb = ConfigCheckButton(
+                _("_Refresh library on start"), "library", "refresh_on_start")
+            cb.set_active(config.getboolean("library", "refresh_on_start"))
+            vb3 = gtk.VBox(spacing=6)
+            vb3.pack_start(hb)
+            vb3.pack_start(cb)
+            f = qltk.Frame(_("Scan _Directories"), child=vb3)
             f.get_label_widget().set_mnemonic_widget(e)
             self.pack_start(f, expand=False)
 
@@ -216,7 +312,7 @@ class PreferencesWindow(qltk.Window):
             e = gtk.Entry()
             e.set_text(config.get("editing", "split_on"))
             e.connect('changed', self.__changed, 'editing', 'split_on')
-            tips.set_tip(e, _('Separators for splitting tags'))
+            e.set_tooltip_text(_('Separators for splitting tags'))
             l = gtk.Label(_("Split _on:"))
             l.set_use_underline(True)
             l.set_mnemonic_widget(e)
@@ -226,14 +322,14 @@ class PreferencesWindow(qltk.Window):
 
             vb2 = gtk.VBox(spacing=0)
             cb = ConfigCheckButton(
-                _("Save ratings and play counts"), "editing", "save_to_songs")
+                _("Save ratings and play _counts"), "editing", "save_to_songs")
             cb.set_active(config.getboolean("editing", "save_to_songs"))
             vb2.pack_start(cb)
             hb = gtk.HBox(spacing=3)
             lab = gtk.Label(_("_Email:"))
             entry = gtk.Entry()
-            tips.set_tip(entry, _("Ratings and play counts will be set for "
-                                  "this email address"))
+            entry.set_tooltip_text(_("Ratings and play counts will be set "
+                                     "for this email address"))
             entry.set_text(config.get("editing", "save_email"))
             entry.connect('changed', self.__changed, 'editing', 'save_email')
             hb.pack_start(lab, expand=False)
@@ -261,8 +357,7 @@ class PreferencesWindow(qltk.Window):
             config.set(section, name, entry.get_text())
 
     def __init__(self, parent):
-        if type(self).__window: return
-        else: type(self).__window = self
+        if self.is_not_unique(): return
         super(PreferencesWindow, self).__init__()
         self.set_title(_("Quod Libet Preferences"))
         self.set_border_width(12)
@@ -277,7 +372,6 @@ class PreferencesWindow(qltk.Window):
         self.show_all()
 
     def __destroy(self):
-        type(self).__window = None
         config.write(const.CONFIG)
 
 
