@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: properties.py e4ffbc653126 2007/07/01 05:32:41 $
 
 import gobject
 import gtk
@@ -25,8 +23,9 @@ class SongProperties(qltk.Window):
                                  gobject.TYPE_NONE, (object,))
                      }
 
-    def __init__(self, library, songs):
+    def __init__(self, library, songs, parent=None):
         super(SongProperties, self).__init__()
+        self.set_transient_for(qltk.get_top_parent(parent))
         if len(songs) > 1: self.set_default_size(600, 400)
         else: self.set_default_size(400, 400)
         self.plugins.rescan()
