@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: quodlibetwindow.py 68c09168ec2f 2009/06/28 06:32:57 $
 
 import os
 import random
@@ -35,7 +33,7 @@ from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.info import SongInfo
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.logging import LoggingWindow
-from quodlibet.qltk.mmkeys import MmKeys
+from quodlibet.qltk.mmkeys_ import MmKeys
 from quodlibet.qltk.msg import ErrorMessage
 from quodlibet.qltk.playorder import PlayOrder
 from quodlibet.qltk.pluginwin import PluginWindow
@@ -148,16 +146,18 @@ class QuodLibetWindow(gtk.Window):
         super(QuodLibetWindow, self).__init__()
         self.last_dir = os.path.expanduser("~")
 
-        tips = qltk.Tooltips(self)
         self.set_title("Quod Libet")
 
-        self.set_default_size(
-            *map(int, config.get('memory', 'size').split()))
+        x, y = map(int, config.get('memory', 'size').split())
+        screen = self.get_screen()
+        x = min(x, screen.get_width())
+        y = min(y, screen.get_height())
+        self.set_default_size(x, y)
         self.add(gtk.VBox())
 
         # create main menubar, load/restore accelerator groups
         self.__library = library
-        self.__create_menu(tips, player, library)
+        self.__create_menu(player, library)
         self.add_accel_group(self.ui.get_accel_group())
 
         accel_fn = os.path.join(const.USERDIR, "accels")
@@ -216,10 +216,9 @@ class QuodLibetWindow(gtk.Window):
         hbox.pack_start(hb, expand=False)
         self.repeat = repeat = qltk.ccb.ConfigCheckButton(
             _("_Repeat"), "settings", "repeat")
-        tips.set_tip(repeat, _("Restart the playlist when finished"))
+        repeat.set_tooltip_text(_("Restart the playlist when finished"))
         hbox.pack_start(repeat, expand=False)
-        self.statusbar = StatusBar()
-        hbox.pack_start(self.statusbar)
+
         align.add(hbox)
         self.child.pack_end(align, expand=False)
 
@@ -247,14 +246,19 @@ class QuodLibetWindow(gtk.Window):
 
         self.child.show_all()
         sw.show_all()
+
+        self.statusbar = StatusBar()
+        hbox.pack_start(self.statusbar)
+        self.statusbar.show()
+
         self.select_browser(
-            self, config.get("memory", "browser"), library, player)
-        self.browser.restore()
-        self.browser.activate()
+            self, config.get("memory", "browser"), library, player, True)
 
         def delayed_song_set():
             song = library.get(config.get("memory", "song"))
-            player.setup(self.playlist, song)
+            seek_pos = config.getint("memory", "seek")
+            config.set("memory", "seek", 0)
+            player.setup(self.playlist, song, seek_pos)
         gobject.idle_add(delayed_song_set)
         self.showhide_playlist(self.ui.get_widget("/Menu/View/SongList"))
         self.showhide_playqueue(self.ui.get_widget("/Menu/View/Queue"))
@@ -269,7 +273,8 @@ class QuodLibetWindow(gtk.Window):
         self.songlist.connect('popup-menu', self.__songs_popup_menu)
         self.songlist.connect('columns-changed', self.__cols_changed)
         self.songlist.connect('columns-changed', self.__hide_headers)
-        self.songlist.get_selection().connect('changed', self.__set_time)
+        self.songlist.get_selection().connect(
+                'changed', util.DeferredSignal(self.__set_time))
 
         library.librarian.connect('removed', self.__set_time)
         library.librarian.connect('added', self.__set_time)
@@ -288,7 +293,9 @@ class QuodLibetWindow(gtk.Window):
             'drag-data-received', QuodLibetWindow.__drag_data_received, self)
 
         self.resize(*map(int, config.get("memory", "size").split()))
-        self.__rebuild(None, False)
+
+        if config.getboolean('library', 'refresh_on_start'):
+            self.__rebuild(None, False)
 
     def __drag_motion(self, ctx, x, y, time):
         # Don't accept drops from QL itself, since it offers text/uri-list.
@@ -352,16 +359,19 @@ class QuodLibetWindow(gtk.Window):
         if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
             self.unmaximize()
         super(QuodLibetWindow, self).hide()
+        map(gtk.Window.hide, qltk.Window.childs)
 
     def present(self):
         super(QuodLibetWindow, self).present()
         if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
             self.maximize()
+        map(gtk.Window.present, qltk.Window.childs)
 
     def show(self):
         super(QuodLibetWindow, self).show()
         if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
             self.maximize()
+        map(gtk.Window.show, qltk.Window.childs)
 
     def destroy(self, *args):
         super(QuodLibetWindow, self).destroy()
@@ -376,7 +386,7 @@ class QuodLibetWindow(gtk.Window):
         if not ssv:
             self.qexpander.set_expanded(True)
 
-    def __create_menu(self, tips, player, library):
+    def __create_menu(self, player, library):
         ag = gtk.ActionGroup('QuodLibetWindowActions')
 
         actions = [
@@ -418,7 +428,7 @@ class QuodLibetWindow(gtk.Window):
             ("View", None, _("_View")),
             ("Help", None, _("_Help")),
             ("OutputLog", gtk.STOCK_EDIT, _("_Output Log"),
-             None, None, lambda *args: LoggingWindow()),
+             None, None, lambda *args: LoggingWindow(self)),
             ]
 
         if "QUODLIBET_DEBUG" in os.environ:
@@ -488,7 +498,8 @@ class QuodLibetWindow(gtk.Window):
             view_actions.append((action, None, label, None, None, i))
         current = browsers.index(config.get("memory", "browser"))
         ag.add_radio_actions(
-            view_actions, current, self.select_browser, (library, player))
+            view_actions, current, self.select_browser,
+            (library, player, False))
 
         for Kind in browsers.browsers:
             if not Kind.in_menu: continue
@@ -508,27 +519,27 @@ class QuodLibetWindow(gtk.Window):
         # Cute. So. UIManager lets you attach tooltips, but when they're
         # for menu items, they just get ignored. So here I get to actually
         # attach them.
-        tips.set_tip(
-            self.ui.get_widget("/Menu/Music/RefreshLibrary"),
-            _("Check for changes in your library"))
-        tips.set_tip(
-            self.ui.get_widget("/Menu/Music/ReloadLibrary"),
-            _("Reload all songs in your library (this can take a long time)"))
-        tips.set_tip(
-            self.ui.get_widget("/Menu/Filters/Top"),
-             _("The 40 songs you've played most (more than 40 may "
-               "be chosen if there are ties)"))
-        tips.set_tip(
-            self.ui.get_widget("/Menu/Filters/Bottom"),
-            _("The 40 songs you've played least (more than 40 may "
-              "be chosen if there are ties)"))
+        self.ui.get_widget("/Menu/Music/RefreshLibrary").set_tooltip_text(
+                _("Check for changes in your library"))
+
+        self.ui.get_widget("/Menu/Music/ReloadLibrary").set_tooltip_text(
+                _("Reload all songs in your library "
+                  "(this can take a long time)"))
+
+        self.ui.get_widget("/Menu/Filters/Top").set_tooltip_text(
+                _("The 40 songs you've played most (more than 40 may "
+                  "be chosen if there are ties)"))
+
+        self.ui.get_widget("/Menu/Filters/Bottom").set_tooltip_text(
+                _("The 40 songs you've played least (more than 40 may "
+                  "be chosen if there are ties)"))
 
     def __browser_configure(self, paned, event, browser):
         if paned.get_property('position-set'):
             key = "%s_pos" % browser.__class__.__name__
             config.set("browsers", key, str(paned.get_relative()))
 
-    def select_browser(self, activator, current, library, player):
+    def select_browser(self, activator, current, library, player, restore):
         if isinstance(current, gtk.RadioAction):
             current = current.get_current_value()
         Browser = browsers.get(current)
@@ -542,6 +553,10 @@ class QuodLibetWindow(gtk.Window):
             self.browser.destroy()
         self.browser = Browser(library, player)
         self.browser.connect('songs-selected', self.__browser_cb)
+        if restore:
+            self.browser.restore()
+            self.browser.activate()
+        self.browser.finalize(restore)
         if self.browser.reordered:
             self.songlist.enable_drop()
         elif self.browser.dropped:
@@ -577,6 +592,7 @@ class QuodLibetWindow(gtk.Window):
 
     def __update_paused(self, player, paused):
         menu = self.ui.get_widget("/Menu/Control/PlayPause")
+        if not menu or not menu.child: return # shutting down
         if paused: key = gtk.STOCK_MEDIA_PLAY
         else: key = gtk.STOCK_MEDIA_PAUSE
         text = gtk.stock_lookup(key)[1]
@@ -644,9 +660,18 @@ class QuodLibetWindow(gtk.Window):
 
     def __jump_to_current(self, explicit):
         if player.playlist.song is None: return
-        if explicit: self.browser.scroll(player.playlist.song)
+
+        if player.playlist.song != self.songlist.model.current and explicit:
+            self.browser.scroll(player.playlist.song)
+
+        #we need to wait until the browser has finished scrolling/filling
+        #and the songlist is ready
+        gobject.idle_add(self.__jump_to_current_in_list, explicit)
+
+    def __jump_to_current_in_list(self, explicit):
         if player.playlist.song == self.songlist.model.current:
             path = self.songlist.model.current_path
+            if path is None: return
             self.songlist.scroll_to_cell(
                 path[0], use_align=True, row_align=0.5)
             if explicit:
@@ -696,7 +721,7 @@ class QuodLibetWindow(gtk.Window):
             self.__make_query("#(playcount < %d)" % (songs[-40] + 1))
 
     def __rebuild(self, activator, force):
-        paths = config.get("settings", "scan").split(":")
+        paths = util.split_scan_dirs(config.get("settings", "scan"))
         exclude = config.get("library", "exclude").split(":")
         copool.add(self.__library.rebuild,
                    paths, self.statusbar.progress, force,
@@ -742,7 +767,7 @@ class QuodLibetWindow(gtk.Window):
             chooser = FileChooser(
                 self, _("Add Music"), formats.filter, self.last_dir)
             cb = None
-        
+
         fns = chooser.run()
         chooser.destroy()
         if fns:
@@ -753,7 +778,7 @@ class QuodLibetWindow(gtk.Window):
             else:
                 added = []
                 self.last_dir = os.path.basename(fns[0])
-                for filename in map(os.path.realpath, fns):
+                for filename in map(os.path.realpath, map(util.fsnative, fns)):
                     if filename in self.__library: continue
                     song = self.__library.add_filename(filename)
                     if song: added.append(song)
@@ -773,7 +798,7 @@ class QuodLibetWindow(gtk.Window):
                     self.browser.activate()
 
         if cb and cb.get_active():
-            dirs = config.get("settings", "scan").split(":")
+            dirs = util.split_scan_dirs(config.get("settings", "scan"))
             for fn in fns:
                 if fn not in dirs: dirs.append(fn)
             dirs = ":".join(dirs)
@@ -789,11 +814,11 @@ class QuodLibetWindow(gtk.Window):
 
     def __current_song_prop(self, *args):
         song = player.playlist.song
-        if song: SongProperties(self.__library.librarian, [song])
+        if song: SongProperties(self.__library.librarian, [song], parent=self)
 
     def __current_song_info(self, *args):
         song = player.playlist.song
-        if song: Information(self.__library.librarian, [song])
+        if song: Information(self.__library.librarian, [song], self)
 
     def __hide_menus(self):
         menus = {'genre': ["/Menu/Filters/FilterGenre",
