@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: remote.py a11c0765b0cf 2008/01/22 06:39:33 $
 
 import os
 import random
@@ -66,7 +64,7 @@ class FIFOControl(object):
             f = os.fdopen(fifo, "r", 4096)
             gobject.io_add_watch(
                 f, gtk.gdk.INPUT_READ, self.__process, *args)
-        except EnvironmentError: pass
+        except (EnvironmentError, AttributeError): pass
 
     def __getitem__(self, key):
         key = key.replace("-", "_")
@@ -108,11 +106,19 @@ class FIFOControl(object):
             player.reset()
         else: player.paused ^= True
 
-    def _focus(self, library, window, player): window.present()
+    def _focus(self, library, window, player):
+        window.show()
+        window.present()
 
     def _volume(self, value, library, window, player):
-        if value[0] == "+": volume = player.volume + 0.05
-        elif value == "-": volume = player.volume - 0.05
+        if value[0] in ('+', '-'):
+            if len(value) > 1:
+                try: change = (int(value[1:]) / 100.0)
+                except ValueError: return
+            else:
+                change = 0.05
+            if value[0] == '-': change = -change
+            volume = player.volume + change
         else:
             try: volume = (int(value) / 100.0)
             except ValueError: return
@@ -226,8 +232,8 @@ class FIFOControl(object):
         if value:
             if value in library: songs = [library[value]]
             else: songs = library.query(value)
-            SongProperties(songs, library, 0)
-        else: SongProperties([player.song], library)
+            SongProperties(songs, library, parent=self)
+        else: SongProperties([player.song], library, parent=self)
 
     def _enqueue(self, value, library, window, player):
         playlist = window.playlist
@@ -291,6 +297,6 @@ class FIFOControl(object):
             f.close()
 
     def _refresh(self, library, window, player):
-        paths = config.get("settings", "scan").split(":")
+        paths = util.split_scan_dirs(config.get("settings", "scan"))
         progress = window.statusbar.progress
         copool.add(library.rebuild, paths, progress, False, funcid="library")
