@@ -4,9 +4,8 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: renamefiles.py 22ebf6069c0a 2008/11/30 19:32:31 $
 
+import os
 import unicodedata
 
 import gtk
@@ -32,6 +31,13 @@ class StripWindowsIncompat(FilterCheckButton):
     _key = "windows"
     BAD = '\:*?;"<>|'
     _order = 1.1
+    def __init__(self):
+        super(StripWindowsIncompat, self).__init__()
+        # If on Windows, force this to be active (and hidden)
+        if os.name == 'nt':
+            self.set_active(True)
+            self.set_sensitive(False)
+            self.set_no_show_all(True)
     def filter(self, original, filename):
         return util.strip_win32_incompat(filename)
 
@@ -97,11 +103,13 @@ class RenameFiles(EditPane):
         rows = [(row[0], row[1], row[2].decode('utf-8')) for row in model]
         for song, oldname, newname in rows:
             try:
-                newname = util.fsencode(newname)
+                newname = util.fsnative(newname)
                 library.rename(song, newname, changed=was_changed)
             except StandardError:
                 if skip_all: continue
-                buttons = (gtk.STOCK_STOP, gtk.RESPONSE_CANCEL,
+                RESPONSE_SKIP_ALL = 1
+                buttons = (_("Ignore _All Errors"), RESPONSE_SKIP_ALL,
+                           gtk.STOCK_STOP, gtk.RESPONSE_CANCEL,
                            _("_Continue"), gtk.RESPONSE_OK)
                 msg = qltk.Message(
                     gtk.MESSAGE_ERROR, win, _("Unable to rename file"),
@@ -115,10 +123,13 @@ class RenameFiles(EditPane):
                 msg.add_buttons(*buttons)
                 msg.set_default_response(gtk.RESPONSE_OK)
                 resp = msg.run()
+                skip_all |= (resp == RESPONSE_SKIP_ALL)
+                # Preserve old behavior: shift-click is Ignore All
                 mods = gtk.gdk.display_get_default().get_pointer()[3]
                 skip_all |= mods & gtk.gdk.SHIFT_MASK
                 library.reload(song, changed=was_changed)
-                if resp != gtk.RESPONSE_OK: break
+                if resp != gtk.RESPONSE_OK and resp != RESPONSE_SKIP_ALL:
+                    break
             if win.step(): break
 
         self.view.thaw_child_notify()
