@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: textedit.py 1d1a43280369 2007/05/18 07:44:07 $
 
 import gobject
 import gtk
@@ -74,12 +72,13 @@ class PatternEditBox(TextEditBox):
             apply.stop_emission('clicked')
         return False
 
-class TextEdit(gtk.Window):
+class TextEdit(qltk.UniqueWindow):
     """A window with a text editing box in it."""
 
     Box = TextEditBox
 
     def __init__(self, parent, default=""):
+        if self.is_not_unique(): return
         super(TextEdit, self).__init__()
         self.set_title(_("Edit Display"))
         self.set_transient_for(qltk.get_top_parent(parent))
@@ -96,4 +95,3 @@ class TextEdit(gtk.Window):
 class PatternEdit(TextEdit):
     """A window with a pattern editing box in it."""
     Box = PatternEditBox
-    
