@@ -3,8 +3,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: uri.py 6919aa22828d 2009/06/22 04:53:02 $
 
 # TODO:
 #  * netloc separation (user/passwd/host/port using urllib)
@@ -54,6 +52,8 @@ class URI(str):
 
     def frompath(klass, value):
         """Construct a URI from an unescaped filename."""
+        # windows unicode path chars may break pathname2url; encode in UTF-8
+        if isinstance(value, unicode): value = value.encode("UTF-8")
         return klass("file://" + pathname2url(value), escaped=True)
     frompath = classmethod(frompath)
 
