@@ -1,5 +1,6 @@
 from tests import TestCase, add
-import browsers
+
+from quodlibet import browsers
 
 class TBrowsers(TestCase):
     def test_presence(self):
@@ -19,10 +20,4 @@ class TBrowsers(TestCase):
     def test_get_invalid(self):
         self.failUnless(
             browsers.get("DoesNotExist") is browsers.search.EmptyBar)
-
-    def test_all(self):
-        for i in range(len(browsers.get_view_browsers())):
-            name = browsers.name(i)
-            self.failUnlessEqual(browsers.get(i), browsers.get(name))
-            self.failUnlessEqual(browsers.index(name), i)
 add(TBrowsers)
