@@ -1,5 +1,6 @@
-from tests import add, TestCase
-from browsers._base import Browser
+from tests import TestCase, add
+
+from quodlibet.browsers._base import Browser
 
 class TBrowser(TestCase):
     def setUp(self):
