@@ -139,6 +139,22 @@ class TFileFromPattern(_TPattern):
         s.assertRaises(ValueError, FileFromPattern, '<a>/<b>')
         FileFromPattern('/<a>/<b>')
 
+    def test_ext_case_preservation(s):
+        x = s.AudioFile({ '~filename':'/tmp/Xx.Flac', 'title':'Xx' })
+        # If pattern has a particular ext, preserve case of ext
+        p1 = FileFromPattern('<~basename>')
+        s.assertEquals(p1.format(x), 'Xx.Flac')
+        p2 = FileFromPattern('<title>.FLAC')
+        s.assertEquals(p2.format(x), 'Xx.FLAC')
+        # If pattern doesn't have a particular ext, lowercase ext
+        p3 = FileFromPattern('<title>')
+        s.assertEquals(p3.format(x), 'Xx.flac')
+
+    def test_backslash_conversion_win32(s):
+        if os.name == 'nt':
+            pat = FileFromPattern(r'Z:\<artist>\<title>')
+            s.assertEquals(pat.format(s.a), 'Z:/Artist/Title5.mp3')
+
 class TXMLFromPattern(_TPattern):
     def test_markup_passthrough(s):
         pat = XMLFromPattern(r'\<b\>&lt;<title>&gt;\</b\>')
@@ -154,6 +170,15 @@ class TXMLFromPattern(_TPattern):
         pat = XMLFromPattern(r'<title|\<b\><title> woo\</b\>>')
         s.assertEquals(pat.format(s.a), '<b>Title5 woo</b>')
 
+class TRealTags(TestCase):
+    def test_empty(self):
+        self.failUnlessEqual(Pattern("").real_tags(), [])
+    def test_both(self):
+        pat = "<foo|<~bar~fuu> - <fa>|<bar>>"
+        self.failUnlessEqual(Pattern(pat).real_tags(), ["foo", "bar", "fuu", "fa"])
+        self.failUnlessEqual(Pattern(pat).real_tags(False), ["bar", "fuu", "fa"])
+
 add(TPattern)
 add(TFileFromPattern)
 add(TXMLFromPattern)
+add(TRealTags)
