@@ -1,106 +1,24 @@
-from unittest import TestCase
-from tests import registerCase
-import os, sys
+from tests import TestCase, add
+
+import os
+
 from tempfile import mkstemp, mkdtemp
-from plugins import PluginManager
 
-class TPluginManager(TestCase):
-
-    def setUp(self):
-        self.tempdir = mkdtemp()
-        self.pm = PluginManager(folders=[self.tempdir])
-        self.pm.rescan()
-        self.assertEquals(self.pm.list(), [])
-
-    def tearDown(self):
-        for f in os.listdir(self.tempdir):
-            os.remove(os.path.join(self.tempdir,f))
-        os.rmdir(self.tempdir)
-
-    def create_plugin(self, name='', desc='', icon='', funcs=None, mod=False):
-        fd, fn = mkstemp(suffix='.py', text=True, dir=self.tempdir)
-        file = os.fdopen(fd, 'w')
-
-        if mod:
-            indent = ''
-        else:
-            file.write("class Foo:\n")
-            indent = '    '
-            file.write("%spass\n" % indent)
-
-        if name: file.write("%sPLUGIN_NAME = %r\n" % (indent, name))
-        if desc: file.write("%sPLUGIN_DESC = %r\n" % (indent, desc))
-        if icon: file.write("%sPLUGIN_ICON = %r\n" % (indent, icon))
-        for f in (funcs or []):
-            file.write("%sdef %s(*args): return args\n" % (indent, f))
-        file.flush()
-        file.close()
-
-    def test_empty_has_no_plugins(self):
-        dirname = self.create_plugin()
-        self.pm.rescan()
-        self.assertEquals(self.pm.list(), [])
-
-    def test_name_only_has_no_plugins(self):
-        dirname = self.create_plugin(name='NameOnly')
-        self.pm.rescan()
-        self.assertEquals(self.pm.list(), [])
-
-    def test_desc_only_has_no_plugins(self):
-        dirname = self.create_plugin(desc='DescOnly')
-        self.pm.rescan()
-        self.assertEquals(self.pm.list(), [])
-
-    def test_name_and_desc_still_none(self):
-        dirname = self.create_plugin(name='Name', desc='Desc')
-        self.pm.rescan()
-        self.assertEquals(self.pm.list(), [])
-
-    def test_name_and_desc_plus_func_is_one(self):
-        self.create_plugin(name='Name', desc='Desc', funcs=['plugin_song'])
-        self.pm.rescan()
-        self.assertEquals(len(self.pm.list()), 1)
-
-    def test_additional_functions_still_only_one(self):
-        self.create_plugin(name='Name', desc='Desc',
-                funcs=['plugin_song', 'plugin_on_changed'])
-        self.pm.rescan()
-        self.assertEquals(len(self.pm.list()), 1)
-
-    def test_two_plugins_are_two(self):
-        self.create_plugin(name='Name', desc='Desc', funcs=['plugin_song'])
-        self.create_plugin(name='Name2', desc='Desc2',
-                funcs=['plugin_on_changed'])
-        self.pm.rescan()
-        self.assertEquals(len(self.pm.list()), 2)
-
-    def test_disables_plugin(self):
-        self.create_plugin(name='Name', desc='Desc', funcs=['plugin_song'])
-        self.pm.rescan()
-        self.failIf(self.pm.enabled(self.pm.list()[0]))
-
-    def test_enabledisable_plugin(self):
-        self.create_plugin(name='Name', desc='Desc', funcs=['plugin_song'])
-        self.pm.rescan()
-        plug = self.pm.list()[0]
-        self.pm.enable(plug, True)
-        self.failUnless(self.pm.enabled(plug))
-        self.pm.enable(plug, False)
-        self.failIf(self.pm.enabled(plug))
+from quodlibet.formats._audio import AudioFile
+from quodlibet.plugins import SongWrapper, ListWrapper
 
 class TSongWrapper(TestCase):
-    from plugins import SongWrapper
-    from formats._audio import AudioFile
 
     psong = AudioFile({
         "~filename": "does not/exist",
         "title": "more songs",
         "discnumber": "2/2", "tracknumber": "1",
-        "artist": "Foo\nI have two artists", "album": "Bar" })
+        "artist": "Foo\nI have two artists", "album": "Bar",
+        "~bookmark": "2:10 A bookmark" })
     pwrap = SongWrapper(psong)
 
     def setUp(self):
-        self.wrap = self.SongWrapper(self.AudioFile(
+        self.wrap = SongWrapper(AudioFile(
             {"title": "woo", "~filename": "/dev/null"}))
 
     def test_slots(self):
@@ -108,7 +26,7 @@ class TSongWrapper(TestCase):
         self.failUnlessRaises(AttributeError, breakme)
 
     def test_cmp(self):
-        songs = [self.SongWrapper(self.AudioFile({"tracknumber": str(i)}))
+        songs = [SongWrapper(AudioFile({"tracknumber": str(i)}))
                  for i in range(10)]
         songs.reverse()
         songs.sort()
@@ -190,5 +108,28 @@ class TSongWrapper(TestCase):
         self.wrap["version"] = "bar"
         self.failUnless(self.wrap._was_updated())
 
-registerCase(TPluginManager)
-registerCase(TSongWrapper)
+    def test_bookmark(self):
+        self.failUnlessEqual(self.psong.bookmarks, self.pwrap.bookmarks)
+        self.pwrap.bookmarks = [(43, "another mark")]
+        self.failUnlessEqual(self.psong["~bookmark"], "0:43 another mark")
+        self.failUnlessEqual(self.psong.bookmarks, self.pwrap.bookmarks)
+
+add(TSongWrapper)
+
+class TListWrapper(TestCase):
+    def test_empty(self):
+        wrapped = ListWrapper([])
+        self.failUnlessEqual(wrapped, [])
+
+    def test_empty_song(self):
+        wrapped = ListWrapper([{}])
+        self.failUnless(len(wrapped) == 1)
+        self.failIf(isinstance(wrapped[0], dict))
+
+    def test_none(self):
+        wrapped = ListWrapper([None, None])
+        self.failUnless(len(wrapped) == 2)
+        self.failUnlessEqual(wrapped, [None, None])
+add(TListWrapper)
+
+        
