@@ -1,9 +1,17 @@
+from tests import TestCase, add
+
 import gtk
-from tests import add, TestCase
-from qltk.ccb import ConfigCheckButton, ConfigCheckMenuItem
-import config
+
+from quodlibet import config
+
+from quodlibet.qltk.ccb import ConfigCheckButton, ConfigCheckMenuItem
 
 class TConfigCheckButton(TestCase):
+    def setUp(self):
+        config.init()
+    def tearDown(self):
+        config.quit()
+
     def test_toggle(self):
         config.set("memory", "bar", "on")
         c = ConfigCheckButton("dummy", "memory", "bar")
@@ -15,6 +23,11 @@ class TConfigCheckButton(TestCase):
 add(TConfigCheckButton)
 
 class TConfigCheckMenuItem(TestCase):
+    def setUp(self):
+        config.init()
+    def tearDown(self):
+        config.quit()
+
     def test_toggle(self):
         config.set("memory", "bar", "on")
         c = ConfigCheckMenuItem("dummy", "memory", "bar")
