@@ -1,8 +1,11 @@
-from tests import add, TestCase
-from qltk.prefs import PreferencesWindow
+from tests import TestCase, add
+
+from quodlibet.qltk.prefs import PreferencesWindow
+import quodlibet.config
 
 class TPreferencesWindow(TestCase):
     def setUp(self):
+        quodlibet.config.init()
         self.win = PreferencesWindow(None)
 
     def test_ctr(self):
@@ -10,4 +13,5 @@ class TPreferencesWindow(TestCase):
 
     def tearDown(self):
         self.win.destroy()
+        quodlibet.config.quit()
 add(TPreferencesWindow)
